function [x,y,z,w,pobj,dobj,it,cgit]= min_congestion(uk,u,b,od,form,BIG,maxit,type_comp_dy)
%
% [x,y,z,w,pobj,dobj,it,cgit]= min_congestion(uk,u,b,od,form,[,BIG,maxit,type_comp_dy])
%
% MIN_CONGESTION solves the minimum congestion problem in multicommodity 
% networks, using the generic prblock_ip solver.
% 
% The minimum congestion problem is:
% 
%   min     max{for all edge e}  y_e
%	subj. to 
%		N xi = bi  i=1,...,k       [Network equations per commodity ]
%		(x1_e+...+ xk_e) <= y_e*u_e for all edge e   [Linking constraints]
%	   0 <= xi <= ui i=1,...,k           [Bounds per commodoty]
%
%     where xi i=1..k are the flows for commodity
%     and y_e the capacity increment factor per edge.
%
% Note: the current code assumes no individual capacities ui per commodity
% 
% The first formulation of the problem (parameter form= 1) is
%    
%   min     z
%	subj. to 
%		N xi = bi  i=1,...,k              
%		0 <= xi <= ui i=1,...,k
%	   (x1_e+...+ xk_e) <= y_e*u_e \  
%                                        = for all edge e (linking constraints)
%           z >= y_e                   /  
%
% The first formulation has a dense column in the second group of 
% linking constraints, which produces fill-in. The second formulation
% avoids this dense column by increasing the number of linking constraints
% and z variables:
%
%   min  z_e_1
%	subj. to 
%		N xi = bi  i=1,...,k              
%		0 <= xi <= ui i=1,...,k  
%	   (x1_e+...+ xk_e) <= y_e*u_e \  
%          z_e >= y_e                   = for all edge e (linking constraints)
%          z_e_i = z_e_{i+1} i=1:n-1   /  
% 
% The code creates a minimum congestion problem from a multicommodity
% problem. Although these problems usually appear in telecomunications problems, 
% with commodities associated to origin-destination pairs, the code 
% formulates the minimum congestion problem of any multicommodity instance.
%
% The input parameters are:
%
% uk[n,k+1]:    upper bounds (capacities) of edges for each commodity; 
%               last column is for slacks of mutual capacity constraints 
% u[n]:         mutual capacity of edges for all commodities
% b[m,k]:       supplies (positive values) and demands (negative values) 
%               for each commodity at nodes of the network
% od[n,2]:      for each edge, origin (first column) and destination 
%               (second column) nodes. The code assumes arcs are
%               nonoriented and for each edge in od[] creates two variables.
% form:         formulation used, 1 or 2
% BIG:          value used as infinity (default is 1.0e+32)
% maxit:        maximum number of IP iterations (default is 200)
% type_comp_dy: scalar sent to prblock_ip that can take values 
%              1: (default value of prblock_ip) dy direction is computed  
%                 with Cholesky for blocks, and PCG for linking constraints 
%              2: dy direction is computed with Cholesky for the full
%                 A*Theta*A' system.
%
% where
% m is the number of nodes in network
% n is number of edges in network
% k is number of commodities
%
% The output parameters are
%
% x[nc]: optimal solution (primal variables): k first columns for flows,
%	     last column for slacks
% y[nr]: optimal dual variables of network and linking constraints
% z[nc]: optimal dual variables of xi >= 0 bounds
% w[nc]: optimal dual variables of xi <= ui bounds
% pobj:     primal objective function
% dobj:     dual objective function
% it:       number of IP iterations performed
% cgit:     overall number of PCG iterations performed 
%
% where nc, number of columns, and nr, number of rows are:
%  if form= 1:
%       nc= (2*n*k)+(1)+(n)+(n+n) 
%           (for x1+, x1-, ..., xk+, xk-,z,y,slacks)
%       nr= (k*m)+(n+n)
%           (for network constraints, and linking constraints)
%  if form= 2:    
%       nc= (2*n*k)+(n)+(n)+(n+n+n-1) 
%           (for x1+, x1-, ..., xk+, xk-,z,y,slacks)%
%       nr= (k*m)+(n+n+n-1)
%           (for network constraints, and linking constraints)
%
% Jordi Castro, June 2005
% Departament of Statistics and Operations Research
% Universitat Politcnica de Catalunya, Barcelona
% www-eio.upc.es/~jcastro, jordi.castro@upc.edu
%

%********** general stuff: check parameters, ... ***********
rhs= nargin;
lhs= nargout;

if (rhs < 5)
   error('too few input parameters');
end
if (rhs >= 5)
  if (form ~=1) & (form ~=2) 
      error('formulation must be type 1 or type 2');
  end
end

DEF_BIG= 1.0e+32;
if (rhs >= 6)
  if (~isscalar(BIG))
      BIG= DEF_BIG;
  end
else
  BIG= DEF_BIG;
end
if (rhs >= 7)
  if (~isscalar(maxit)) 
    maxit= [];
  end
else
  maxit= [];
end
if (rhs >= 8) 
  if (~isscalar(maxit)) 
    type_comp_dy= [];
  end
else
  type_comp_dy= [];
end

% simple preprocess for mmcnf problem
% changes uk and u
[m,n,k,uk,u]= mincong_preprocessing(uk,u,b,od,BIG);

% create minimum_congestion problem from mmcnf problem
[nc,nr,c2,u2,b2,N,L]= mmcnf2mincong(m,n,k,uk,u,b,od,form,BIG);

% solve problem through generic solver
[x,y,z,w,pobj,dobj,active,it,cgit]= ...
    prblock_ip(c2,[],u2,b2,N,L,[],BIG,maxit,type_comp_dy);

 

function [m,n,k,uk,u]= mincong_preprocessing(uk,u,b,od,BIG)
%
% Performs a simple preprocessing of the problem
%
% Output parameters:
% - m,n,k: number of nodes, arcs and commodities
% - uk,u: without zero capacities, if needed
%
% Simple check for
% - dimensions
% - no-zero upper bounds (arcs and slacks) not allowed. Set a very small
%   upper bound to guarantee an interior solution. Zero arcs are not removed
%   to preserve he topology of the network for all the commodities,
%   then we can maintain one single symbolic factorization.
% - nodes in network topology between bounds
%

%Dimensions

[n,k1]= size(uk);
k= k1-1;
[nr]= size(u);
if (nr ~= n )  error('inconsistent dimensions: u'); end
[m,nc]= size(b);
if (nc ~= k )  error('inconsistent dimensions: b'); end
[nr,nc]= size(od);
if (nr ~= n ) | (nc ~= 2)  error('inconsistent dimensions: od'); end

% Network topology
if any(any(od<1 | od>m))  error('node<1 or node>m  in network topology'); end

% zero capacities replaced by small value (EPS_CAP)
EPS_CAP= 1.0e-16;
uk(uk<EPS_CAP)= EPS_CAP;
u(u<EPS_CAP)= EPS_CAP;

% Infinity capacities are replaced by the maximum of the maximum capacity
% (excluding infinity capacities) and an upper bound of the flow at each arc.
% This can provide a wrong problem for networks with negative costs,
% infinity capacities, and negative cycles. These problems should be avoided.
% This has to be done BEFORE detecting inactive mutual capacity constraints,
% because the original infinity capacity might reduce the number of
% inactive constraints.
maxflow= sum(b(b>0));
% epsilon used when u()= BIG (i.e., k=1 and all linking constraints inactive)
EPSILON= 1.0e-10;
maxcap= max(max(uk(uk(:,1:k)<BIG)),max([u(u<BIG);EPSILON]));
newinfcap= max(maxflow,maxcap);
uk(uk(:,1:k) >= BIG)= newinfcap;
u(u >= BIG)= newinfcap;


function [nc,nr,c2,u2,b2,N,L]= mmcnf2mincong(m,n,k,uk,u,b,od,form,BIG);
% creates minimum congestion problem from mmcnf problem


% common part for form=1 or 2
% injections of minimum congestion problem
t= 2; % increment of injections factor
b2= t*reshape(b(1:m-1,1:k),k*(m-1),1);
% bounds on arcs, replicating capacities for backward arcs
for i=1:k
  u2(:,2*(i-1)+1)= uk(:,i);
  u2(:,2*(i-1)+2)= uk(:,i);
end
u2= reshape(u2,2*k*n,1);
%**
u2= BIG*ones(2*k*n,1);


% nonoriented network and mutual capacity constraints blocks
Nnon= mincong_network_matrix(od,1);
Inon= [speye(n,n) speye(n,n)];
%
N= cell(k+2,1);
L= cell(k+2,1);
for i=1:k
   N{i}= Nnon;
end


% specific part depending on form=1 or 2 
% BIG used for bounds on z,y and slacks of mutual capacity and z <= y 
if (form==1)
    nc= (2*n*k)+(1)+(n)+(n+n); % for x1+, x1-, ..., xk+, xk-,z,y,slacks
    nr= (k*(m-1))+(n+n); % for network constraints, and linking constraints
    N{k+1}= sparse(0,1); %z
    N{k+2}= sparse(0,n); %y
    for i=1:k
       L{i}= [Inon; sparse(n,2*n)];
    end
    L{k+1}= [sparse(n,1); sparse(-ones(n,1))]; %z [0;-e]
    L{k+2}= [-diag(sparse(u)); speye(n,n)]; %y [-U; I]
    % 
    b2= [b2;zeros(n+n,1)]; %rhs of linking constraints is 0
    %
    c2= zeros(nc,1);
    c2(2*n*k+1)= 1; % position of z
    %
    u2= [u2 ; BIG*ones(1+n,1); BIG .*ones(n,1); BIG*ones(n,1)];

else % form=2
    nc= (2*n*k)+(n)+(n)+(n+n+n-1); % for x1+, x1-, ..., xk+, xk-,z,y,slacks
    nr= (k*(m-1))+(n+n+n-1); % for network constraints, and linking constraints
    N{k+1}= sparse(0,n); %z
    N{k+2}= sparse(0,n); %y
    for i=1:k
       L{i}= [Inon; sparse(n,2*n);sparse(n-1,2*n)];
    end
    %** shiftmatrix= speye(n-1,n-1);
    shiftmatrix= 1*speye(n-1,n-1);
    for i=1:n-1
      %**shiftmatrix(i,i+1)= -1;
      shiftmatrix(i,i+1)= -1*1;
    end
    L{k+1}= [sparse(n,n); -speye(n,n); shiftmatrix]; %z [0;-I;zei-zei+1=0 matrix]
    L{k+2}= [-diag(sparse(u)); speye(n,n);sparse(n-1,n)]; %y [-U; I;0]
    % 
    b2= [b2;zeros(n+n+(n-1),1)]; %rhs of linking constraints is 0
    %
    c2= zeros(nc,1);
    c2(2*n*k+1)= 1; % position of z_e1
    %
    % force slack of equality constraints is almost 0
    u2= [u2 ; BIG*ones(n+n,1); BIG .*ones(n,1); BIG*ones(n,1); 1.0e-6*ones(n-1,1)];
end


function [N]= mincong_network_matrix(od,flag)
%
% Creates the nonoriented node-arc incidence matrix
% 
% Input parameters:
% -od[1:n,2]: origin and destination for each arc
% -flag: if is not zero, we remove the last node (last row), providing a 
%  full row rank matrix; if is zero, a rank defficient matrix that includes
%  last row is returned.

% Output parameters:
% - N: sparse matrix with node-arc incidence network matrix

% compute number of arcs n, and number of nodes
% it is assumed that the number of nodes is the maximum value in od
[n,nc]= size(od);
m= max(max(od));

% ij[1:n,1:2] store the location (row,col) of the +1
ij(1:n,2)= [1:n]';
ij(1:n,1)= od(1:n,1);
% ij[n+1:2*n,1:2] store the location (row,col) of the -1
ij(n+1:2*n,2)= [1:n]';
ij(n+1:2*n,1)= od(1:n,2);
% v[1:2*n] store the values
v(1:n)= +1;
v(n+1:2*n)= -1;
% create the network
N= sparse(ij(:,1),ij(:,2),v); 
if (flag ~=0) 
  N= N(1:m-1,1:n);
end
N= [N -N]; 

 
