function [x,y,z,w,pobj,dobj,it,cgit]= nonor_qipm(c,q,uk,u,b,od,BIG,maxit,type_comp_dy)
%
% [x,y,z,w,pobj,dobj,it,cgit]= nonor_qipm(c,q,uk,u,b,od[,BIG,maxit,type_comp_dy])
%
% NONOR_QIPM implements a specialized primal-dual long-step path-following
% interior point algorithm for nonoriented linear or quadratic minimum cost 
% multicommodity network flow problem:
%
%      min     c1 (x1+ + x1-) + ...+ ck (xk+ + xk-) + ck+1 xk+1 + 
%              1/2 (x1+ x1- ... xk+1) Q (x1+ x1- ... xk+1)'
%	subj. to 
%		N xi+ - N xi- = bi  i=1,...,k  Network equations per commodity 
%		(x1+ + x1- + ...+ xk+ + xk-) + xk+1 = u   Linking constraints
%	   0 <= xi+,xi- <= ui   i=1,...,k      Arcs Bounds
%          0 <= xk+1 <= uk+1                  Slacks bounds  
%
%     where xi+ / xi- i=1..k are the forward / backward flows for commodity
%     and xk+1 are the slacks of the linking constraints
% 
% A description of the original algorithm for the oriented 
% linear case can be found in
%	J. Castro, A specialized interior-point algorithm for 
%	multicommodity network flows , SIAM Journal on 
%	Optimization, 10 (2000), pp. 852-877.
% 
% It calls the generic primal block solver prblock_ip
%
% The current implementations extends the algorithm of the above paper:
% - it deals with nonoriented networks
% - it deals with quadratic costs
% - it considers linear and quadratic costs for slacks of linking constraints
% - it considers lower and upper bounds for linking constraints
%
% The input parameters are:
%
% c[n,k+1]:     linear forward arc costs; last column is for slacks of 
%               mutual capacity constraints. The same arc costs are
%               assumed for the backward arcs.
% q[n,k+1]:     quadratic forward arc costs; last column is for slacks of 
%               mutual capacity constraints (q=[] if no quadratic costs).
%               The same costs are assumed for backward arcs.
% uk[n,k+1]:    upper bounds (capacities) of forward arcs for each commodity; 
%               last column is for slacks of mutual capacity constraints.
%               The same capacities are assumed for backward arcs.
% u[n]:         mutual capacity of arcs for all commodities.
% b[m,k]:       supplies (positive values) and demands (negative values) 
%               for each commodity at nodes of the network
% od[n,2]:      for each forward arc, origin (first column) and destination 
%               (second column) nodes
% BIG:          value used as infinity (default is 1.0e+32)
% maxit:        maximum number of IP iterations (default is 200)
% type_comp_dy: scalar sent to prblock_ip that can take values 
%              1: (default value of prblock_ip) dy direction is computed with 
%                 Cholesky for blocks, and PCG for linking constraints 
%              2: dy direction is computed with Cholesky for the full
%                 A*Theta*A' system.
%
% where
% m is the number of nodes in network
% n is number of forward arcs in network
% k is number of commodities
%
% The output parameters are
%
% x[n,2*k+1]: optimal solution (primal variables). The column order is:
%	       x1+  x1-  ... xk+  xk-  xk+1
% y[k*m+n]: optimal dual variables of network and linking constraints
% z[n,2*k+1]: optimal dual variables of xi >= 0 bounds. 
%	       Same column order that x.
% w[n,2*k+1]: optimal dual variables of xi <= ui bounds. 
%	       Same column order that x.
% pobj:     primal objective function
% dobj:     dual objective function
% it:       number of IP iterations performed
% cgit:     overall number of PCG iterations performed 
%
% Jordi Castro, June 2005
% Departament of Statistics and Operations Research
% Universitat Politcnica de Catalunya, Barcelona
% www-eio.upc.es/~jcastro, jordi.castro@upc.edu
%

%********** general stuff: check parameters, ... ***********
rhs= nargin;
lhs= nargout;

if (rhs < 6)
   error('too few input parameters');
end
DEF_BIG= 1.0e+32;
if (rhs >= 7)
  if (~isscalar(BIG))
      BIG= DEF_BIG;
  end
else
  BIG= DEF_BIG;
end
if (rhs >= 8)
  if (~isscalar(maxit)) 
    maxit= [];
  end
else
  maxit= [];
end
if (rhs >= 9) 
  if (~isscalar(maxit)) 
    type_comp_dy= [];
  end
else
  type_comp_dy= [];
end

% ********* preprocess mutual capacity constraints ************
% uk is modified by qipm_preprocessing (only upper bounds of arcs)
% u is also modified
[m,n,k,uk,u,activecmt]= qipm_preprocessing(c,q,uk,u,b,od,BIG);

% compute nonoriented node-arc incidence matrix N, excluding last row (flag~=0)
flag= 1;
N= qipm_nonor_network_matrix(od,flag);

% compute linking constraints structure matrix
L= qipm_nonor_linking_matrix(n);

% replicate costs, and capacities for backward arcs
for i=1:k
  c2(:,2*(i-1)+1)= c(:,i);
  c2(:,2*(i-1)+2)= c(:,i);
  if (~isempty(q))
      q2(:,2*(i-1)+1)= q(:,i);
      q2(:,2*(i-1)+2)= q(:,i);
  end
  uk2(:,2*(i-1)+1)= uk(:,i);
  uk2(:,2*(i-1)+2)= uk(:,i);
end
c2(:,2*k+1)= c(:,k+1);
if (~isempty(q))
    q2(:,2*k+1)= q(:,k+1);
end
uk2(:,2*k+1)= uk(:,k+1);


% empty_activecmt selects if activecmt= [] must be sent to prblock_ip
empty_activecmt= 1;

if (empty_activecmt) 
  % reshape problem matrices to vectors, avoiding injections of last node
  % and considering only active linking constraints (activecmt2= []);
  c2= [reshape(c2(1:n,1:2*k),k*2*n,1) ; c2(activecmt,2*k+1)];
  if (~isempty(q))  
    q2= [reshape(q2(1:n,1:2*k),k*2*n,1) ; q2(activecmt,2*k+1)];
  else
    q2= [];
  end
  u2= [reshape(uk2(1:n,1:2*k),k*2*n,1); uk2(activecmt,2*k+1)];
  b2= [reshape(b(1:m-1,1:k),k*(m-1),1); u(activecmt)];
  L2=L(activecmt,:);
  activecmt2= [];
else 
  % reshape problem matrices to vectors, avoiding injections of last node
  % and considering all linking constraints
  c2= reshape(c2,k*2*n+n,1);
  if (~isempty(q))  
    q2= reshape(q,k*2*n+n,1);
  else
    q2= [];
  end
  u2= reshape(uk,k*2*n+n,1);
  b2= [reshape(b(1:m-1,1:k),k*(m-1),1); u];
  L2= L;
  activecmt2= activecmt;
end

% solve problem through generic solver
[x2,y2,z2,w2,pobj,dobj,activecmt2,it,cgit]= ...
    prblock_ip(c2,q2,u2,b2,N,L2,activecmt2,BIG,maxit,type_comp_dy);

% reshape solution vectors, acording to empty_activecmt
% adding positions of y associated to last node with a 0 value

if (empty_activecmt) 
  % only positions of active linking constraints were considered
  l= size(L2,1);
  y= zeros(k*m+n,1);
  for i=1:k
    y((i-1)*m+1:i*m-1)= y2((i-1)*(m-1)+1:i*(m-1));
    y(i*m)= 0;
  end
  ycmt= zeros(n,1);
  ycmt(activecmt)= y2(k*(m-1)+1:k*(m-1)+l);
  y(k*m+1:k*m+n)= ycmt;
  
  x= reshape(x2(1:2*n*k),n,2*k);
  x(1:n,2*k+1)= 0;
  x(activecmt,2*k+1)= x2(2*k*n+1:2*k*n+l);

  z= reshape(z2(1:2*n*k),n,2*k);
  z(1:n,2*k+1)= 0;
  z(activecmt,2*k+1)= z2(2*k*n+1:2*k*n+l);

  w= reshape(w2(1:2*n*k),n,2*k);
  w(1:n,2*k+1)= 0;
  w(activecmt,2*k+1)= w2(2*k*n+1:2*k*n+l);
else
  % positions for all linking constraints were considered
  for i=1:k
    y((i-1)*m+1:i*m-1)= y2((i-1)*(m-1)+1:i*(m-1));
    y(i*m)= 0;
  end
  y(k*m+1:k*m+n)= y2(k*(m-1)+1:k*(m-1)+n);

  x= reshape(x2,n,2*k+1);
  z= reshape(z2,n,2*k+1);
  w= reshape(w2,n,2*k+1);
end
 

function [m,n,k,uk,u,activecmt]= qipm_preprocessing(c,q,uk,u,b,od,BIG)
%
% Performs a simple preprocessing of the problem (see below an explanation)
%
% Output parameters:
% - m,n,k: number of nodes, nonoriented arcs and commodities
% - uk[m,k+1]: updated vector of capacities (only changes uk(1:m,1:k)
% - activecmt(): vector with the indices of active mutual capacity
%   constraints; indices correspond to the relative numeration of
%   mutual capacity constraints: 1:n
%
% Simple check for
% - dimensions
% - no-zero upper bounds (arcs and slacks) not allowed. Set a very small
%   upper bound to guarantee an interior solution. Zero arcs are not removed
%   to preserve he topology of the network for all the commodities,
%   then we can maintain one single symbolic factorization.
% - nodes in network topology between bounds
%
% Simple preprocessing for
% - infinity capacities, replaced by an upper bound of the flow in the arc.
%   This has to be done BEFORE preprocessing active linking constraints, since
%   infinity capacities might reduce the number of inactive constraints.
% - linking constraints: early detection of inactive 
%   constraints; they are dinamycally updated later during the optimization
%   in order to reduce the size of the system to be solved by PCG

%Dimensions
[n,k1]= size(c);
k= k1-1;
if (~isempty(q)) 
  [nr,nc]= size(q);
  if (nr ~= n ) | (nc ~= k1) error('inconsistent dimensions: q'); end
end
[nr,nc]= size(uk);
if (nr ~= n ) | (nc ~= k1) error('inconsistent dimensions: uk'); end
[nr]= size(u);
if (nr ~= n )  error('inconsistent dimensions: u'); end
[m,nc]= size(b);
if (nc ~= k )  error('inconsistent dimensions: b'); end
[nr,nc]= size(od);
if (nr ~= n ) | (nc ~= 2)  error('inconsistent dimensions: od'); end

% Network topology
if any(any(od<1 | od>m))  error('node<1 or node>m  in network topology'); end

% zero capacities replaced by small value (EPS_CAP)
EPS_CAP= 1.0e-16;
uk(uk<EPS_CAP)= EPS_CAP;
u(u<EPS_CAP)= EPS_CAP;

% Infinity capacities are replaced by the maximum of the maximum capacity
% (excluding infinity capacities) and an upper bound of the flow at each arc.
% This can provide a wrong problem for networks with negative costs, 
% infinity capacities, and negative cycles. These problems should be avoided.
% This has to be done BEFORE detecting inactive mutual capacity constraints,
% because the original infinity capacity might reduce the number of
% inactive constraints.
maxflow= sum(b(b>0));
% epsilon used when u()= BIG (i.e., k=1 and all linking constraints inactive)
EPSILON= 1.0e-10;
maxcap= max(max(uk(uk(:,1:k)<BIG)),max([u(u<BIG);EPSILON]));
newinfcap= max(maxflow,maxcap);
uk(uk(:,1:k) >= BIG)= newinfcap;

% Early detection of inactive linking constraints, checking whether:
% - sum of capacities for all commodities lower than mutual capacity 
% We don't treat the particular case where k=1 (single commodity problem).
% In that case the mutual capacity u() and the upper bound of slacks
% uk(:,k+1) should be set to BIG by the user, and the slacks linear and
% quadratic costs to 0. That way, all the mutual capacity constraints
% will be automatically considered inactive, and removed.
%
% This check is only performed if:
% 1. the linking constraint of the arc has
% only upper bounds u and the default lower bound is 0, i.e., if the upper 
% bound of the slack is equal to u. 
% 2. the slack of the linking constraint does not appear in the objective 
% function; otherwise, removing this constraint (and slack) can provide
% bad function evaluations. 
% docheck is used for this purpose.
if (isempty(q))
   docheck= (u==uk(:,k+1)) & (c(:,k+1)== 0);
else
   docheck= (u==uk(:,k+1)) & (c(:,k+1)== 0) & (q(:,k+1)== 0);
end
% The list of active linking mutual capacity constraints is maintained by 
% vector activecmt(i)= True if i active, False if i inactive, i=1..n
% We compute an equivalent vector inactivecmt for inactive constraints, 
% and complement it at the end
%
% Check if sum of capacities of all commodities is lower than mutual capacity
% or if mutual capacity is greater or equal than BIG
cap_all_comm= sum(uk(:,1:k),2);
inactivecmt= ( (cap_all_comm<=u) & docheck) | ((u>=BIG) & docheck);
activecmt= (find(~inactivecmt))';
 

function [N]= qipm_nonor_network_matrix(od,flag)
%
% Creates the nonoriented node-arc incidence matrix
% 
% Input parameters:
% -od[1:n,2]: origin and destination for each arc
% -flag: if is not zero, we remove the last node (last row), providing a 
%  full row rank matrix; if is zero, a rank defficient matrix that includes
%  last row is returned.

% Output parameters:
% - N: sparse matrix with nonoriented node-arc incidence network matrix

% compute number of arcs n, and number of nodes
% it is assumed that the number of nodes is the maximum value in od
[n,nc]= size(od);
m= max(max(od));

% ij[1:n,1:2] store the location (row,col) of the +1
ij(1:n,2)= [1:n]';
ij(1:n,1)= od(1:n,1);
% ij[n+1:2*n,1:2] store the location (row,col) of the -1
ij(n+1:2*n,2)= [1:n]';
ij(n+1:2*n,1)= od(1:n,2);
% v[1:2*n] store the values
v(1:n)= +1;
v(n+1:2*n)= -1;
% create the network
N= sparse(ij(:,1),ij(:,2),v); 
if (flag ~=0) 
  N= N(1:m-1,1:n);
end
N= [N -N]; 

function [L]= qipm_nonor_linking_matrix(n)
%
% Creates the linking matrix constraints. For nonoriented multicommodity flows
% is [I I] where I is the the identity matrix of dimension n
% The input parameter:
% n: number of arcs
% 
L= speye(n,n);
L= [L L];
