function [x,y,z,w,pobj,dobj,activelnk,it,cgit]= ...
      prblock_ip(c,q,u,b,N,L,activelnk,BIG,maxit,type_comp_dy)
%
% [x,y,z,w,pobj,dobj,activelnk,it,cgit]= ...
%              prblock_ip(c,q,u,b,N,L,[,activelnk,BIG,maxit,type_comp_dy])
%
% prblock_ip implements a specialized primal-dual long-step path-following
% interior point algorithm for linear and separable quadratic problems 
% with primal block structure:
%
%      min     c_1 x_1 + ...+ c_k x_k + c_{k+1} x_{k+1} + 
%              1/2 (x_1...x_k x_{k+1}) Q (x_1...x_k x_{k+1})
%	subj. to 
%	   N_i x_i = b_i  i=1,...,k                       Block equations  
%	   (L_1*x_1 + ...+ L_k*x_k) + x_{k+1} = b_{k+1}   Linking constraints
%	   0 <= x_i <= u_i   i=1,...,k+1                  Bounds
%
%     where x_i i=1..k are the variables for each block
%     and x_{k+1} are the slacks of the linking constraints
%
% The normal equations for the dual direction is computed in two parts. 
% The part associated to blocks is solved through k Cholesky factorizations.
% The part associated to linking constraints is computed with a PCG  
% using a power series preconditioner. Cholesky 
% factorizations are performed with the Ng-Peyton sparse package.
% 
% This algorithm was originally introduced for linear MMCNF problem in
%	J. Castro, A specialized interior-point algorithm for 
%	multicommodity network flows , SIAM Journal on 
%	Optimization, 10 (2000), pp. 852-877.
%
% This implementation extends the algorithm of the above paper in
% the following aspects:
% - it can deal with any problem with primal block-angular structure, not only MMNCF
% - it deals with quadratic costs of variables
% - it considers linear and quadratic costs for slacks of linking constraints
% - it considers lower and upper bounds for linking constraints
%
% The use can also solve the dual direction by the Cholesky factorization
% of the full system A*Theta*A' (A includes block and linking constraints).
% Which form is used for dy is governed by argument type_comp_dy (see below).
%
%
% Denoting as 
%
% m_i : the number of constraints of block i  
% n_i : number of variables of block i
% k : number of blocks
% l : number of linking constraints (and slacks)
%
% -the number of variables is kn+l, where kn= k*n if all blocks are
%       the same, or kn= sum{i=1..k} n_i if blocks are different.
% - the number of constraints is km+l, where km= k*m if all blocks are
%       the same, or km= sum{i=1..k} m_i if blocks are different.
%
% Vectors of dimension kn+l have first the values for variables of 
% block1, ...,block k, and in last positions the values of 
% slacks of linking constraints.
%
% Vectors of dimension km+l have first the values for constraints of
% block1, ...,block k, and in last positions the values of 
% linking constraints.
%
% The input arguments are
%
% c[kn+l]:     linear costs of variables and slacks of linking constraints
% q[kn+l]:     quadratic costs of variables and slacks of linking constraints
%               (q=[] if no quadratic costs)
% u[kn+l]:     upper bounds of variables and slacks of linking constraints.
% b[km+l]:     right-hand-side of block and linking constraints
% N:            sparse matrix N[m,n] with the structure of block 
%               constraints for all the blocks;
%               or cell array of k matrices of dimension [m_i,n_i],
%               if block constraints are different for each block.
%               For some i, m_i=0 is allowed in Matlab (not in
%               scilab; use a 0 row instead)
% L:            sparse matrix L[l,n] with the structure of linking 
%               constraints for all the blocks; 
%               or cell array of k matrices of dimension [l,n_i], if 
%               linking constraints are different for each block.
%
% Note that zero upper bounds are not allowed; instead use a very small 
% (almost zero) value
%
% The next arguments are optional:
% 
% activelnk[]: vector of indices of active linking constraints
%              (default is all linking constraints active);
%              indices correspond to the relative numbering within
%              linking constraints 1:l. Shift km or kn is added 
%              to access active constraints or slacks of active constraints
%              with the absolute numbering of constraints and variables
% BIG:         value used as infinity (default is 1.0e+32)
% maxit:       maximum number of IP iterations (default is 200)
% type_comp_dy:scalar that can take values 
%              1: (default value) the dy direction is computed with 
%                 Cholesky for blocks, and PCG for linking constraints 
%              2: the dy direction is computed with Cholesky for the full
%                 A*Theta*A' system.
%              3: the dy direction is computed with PCG using as
%                 preconditioner an incomplete Cholesky factorization
%                 of normal equations A*Theta*A' (permuted to avoid fill-in).
%              4: the dy direction is computed as for type_comp_dy=1 but
%                 using as preconditioner an incomplete Cholesky factorization
%                 of the D matrix of the Schur complement S of A*Theta*A' 
%                 (S= D -C'*B^{-1}C). Note that this option can provide  
%                 the same results that type_comp_dy= 1
%                 if only one term of the power series is considered
%                 and the Cholesky and incomplete Cholesky factorizations of
%                 D are the same (for instance, for diagonal D).
%
%              5: the dy direction is computed as for type_comp_dy=1 but
%                 using as preconditioner an incomplete Cholesky factorization
%                 of  D-C' diag(B)^{-1}C, where C,D,B are submatrices of 
%                 the Schur complement S of A*Theta*A' (S= D -C'*B^{-1}C). 
%                 
%
% The output parameters are
%
% x[kn+l]: optimal solution (primal variables); last column for slacks
% y[km+l]: optimal dual variables of block and linking constraints
% z[kn+l]: optimal dual variables of x_i >= 0 bounds
% w[kn+l]: optimal dual variables of x_i <= u_i bounds
% pobj:     primal objective function
% dobj:     dual objective function
% activelnk[]: vector of indices of active linking constraints
%              indices correspond to the relative numbering 1:l 
% it:       number of IP iterations performed
% cgit:     overall number of PCG iterations performed 
%
% Jordi Castro
% Departament of Statistics and Operations Research
% Universitat Politcnica de Catalunya, Barcelona
% www-eio.upc.es/~jcastro, jordi.castro@upc.edu
%
% June 2005

%********** check parameters, load some stuff, ... ***********

cpu_total= cputime;

x=[]; y=[]; z=[]; w=[]; cx=[];

lhs= nargout;
rhs= nargin;

if (rhs < 6)
   error('too few input parameters');
end
if (rhs==6) 
  activelnk=[]; 
end

% don't check the first 7 arguments, and don't perform a careful check
% of the next ones; we expect a careful user :-)
%
% activelnk dealt with later in routine pbip_preprocessing
%

DEF_BIG= 1.0e+32;
if (rhs >= 8)
  if (~isscalar(BIG))
    BIG= DEF_BIG;
  end
else
  BIG= DEF_BIG;
end
DEF_MAXIT= 200;
if (rhs >= 9) 
  if (~isscalar(maxit))
    maxit= DEF_MAXIT;
  end
else
  maxit= DEF_MAXIT;
end
DEF_COMP_DY= 1;
if (rhs >= 10) 
  if (~isscalar(type_comp_dy))
    type_comp_dy= DEF_COMP_DY;
  elseif ((type_comp_dy <1) | (type_comp_dy > 5))  
    error('wrong value: type_comp_dy');    
  end		 
else
  type_comp_dy= DEF_COMP_DY;
end

%********** initializations for optimization ***********     

% declare and initialize some global and general data
% for Ng-Petyon sparse Cholesky structure
% spchoN: tlist with all data for factorization of N*N'
% all_lnzN: stores factorization of N*Theta_i*N' for all blocks i=1..k
% spchoD: tlist with all data for factorization of D= (sum{i=1:k}L*L')+I
%         the Schur complement part associated to linking constraints
% spchoA: tlist with all data for factorization of full A matrix, with 
%         block and linking constraints
global spchoN all_lnzN spchoD; 
global OPTGAP PFEAS DFEAS RHO SIGMA pcgtol;
OPTGAP= 1.0e-4; % gap optimality tolerance at optimum
PFEAS= 1.0e-6; % primal feasibility tolerance at optimum
DFEAS= 1.0e-6; % dual feasibility tolerance at optimum
RHO= 0.995;  % step reduction
SIGMA= 0.1; % centrality reduction
global m_pw_prec; % number of terms in the power series expansion of H^{-1}
m_pw_prec= 1;
global m n k l km kn; % problem dimensions
global ini_m ini_n; % initial row and column per block (if N is list of matrices)
global it; % iteration counter
% the following booleans are used to
% know if the symbolic factorization of N*N', A*A' and D, or the permutation
% of A*A' for full PCG, were
% already made; this way, we can swith from one type_comp_dy to another
% during the iterative optimization
global made_symbfct_NN made_symbfct_D made_symbfct_AA made_perm_AA;
global made_perm_D;
global made_perm_DCBC;
made_symbfct_NN= logical(0);
made_symbfct_D= logical(0);
made_symbfct_AA= logical(0);
made_perm_AA= logical(0);
made_perm_D= logical(0);
made_perm_DCBC= logical(0);
% stores full matrix A
global A;
% stores permutation when comp_dir_dy is 3
global perm_AA;
% stores permutation and IC when comp_dir_dy is 4
global perm_D;
global RD;
% stores permutation and IC when comp_dir_dy is 5
global perm_DCBC;
global RDCBC;
% stores cputime of Ng-Peyton compiled routines: for symbolic factorization,
% numerical factorization, and solution
global cpu_symfct cpu_numfct cpu_solfct;
cpu_symfct= 0;
cpu_numfct= 0;
cpu_solfct= 0;
% cpu of preconditioner in my_pcg
global cpu_my_pcg_precond
% cputime when incomplete Cholesky factorization used 
% instead of Cholesky (i.e., for type_comp_dy=3,4,5)
global cpu_mindegreeord cpu_cholinc cpu_pcg_cholinc;
cpu_mindegreeord= 0;
cpu_cholinc= 0;
cpu_pcg_cholinc= 0;
% boolean for switching to incomplete Cholesky based on droptolerance
% (if switch_droptol is true) or based on 0 fill-in (which seems to exist
% for the first iterations but from some point does not; this is something
% to study in the future). If incomplete Cholesky based on drop-tolerance
% the droptol_cholinc value will be used; and is updated decreasing its
% value as we approach the solution (to guarantee a better factor, and less
% PCG iterations in last difficult IP iterations).
global droptol_cholinc switch_droptol;
droptol_cholinc= 1.0e-3;
switch_droptol= logical(0);
% set of variables with and without upper bounds (i.e., u<BIG u>=BIG)
global set_with_ub set_without_ub;

% q is set to 0 if empty
% activelnk set to indices of all linking constraints (i.e., 'all active')
% if initially empty
[m,n,k,l,km,kn,ini_m,ini_n,q,activelnk]= pbip_preprocessing(c,q,u,b,N,L,activelnk,BIG);

% set precision of solution for PCG
if (type_comp_dy==1) || (type_comp_dy>=4)
    if (all(q == 0) )  %linear problem
      pcgtol= 1.0e-2; 
    else
    pcgtol= 1.0e-3;
    end
elseif (type_comp_dy==3)
    pcgtol= 1.0e-3;
end

% compute norm of b and c, used all over the iterative procedure
[normb,normc]= pbip_normb_normc(b,c,activelnk);

% compute initial point
initype= 1;
[x,y,z,w]= pbip_initial_point(N,c,q,u,activelnk,initype);

% get variables s at initial point and compute initial KKT residuals
% (infeasibility and optimality)
s= pbip_get_s(u,x,activelnk);
[rb,rc,rxz,rsw,normrb,normrc,mu] = ...
    pbip_KKT_residuals(x,y,z,s,w,activelnk,b,c,q,normb,normc,N,L);

% primal and dual objective function at current point
[pobj,dobj,gap]= pbip_objective_functions(b,c,q,u,x,y,w,activelnk);

it= 0; % counter for iterations
cgit= 0; % counter for pcg iterations

% save original type_comp_dy for right CPU info at the end of algorithm
orig_type_comp_dy= type_comp_dy;

%********** start iterative optimization***********     

% just focus on optimality gap to avoid numerical inestabilities due to PCG
% while ( ((gap>OPTGAP) | (normb > PFEAS) | (normc > DFEAS)) & (it < maxit) )
while ( (gap > OPTGAP) & (it < maxit) )  

  %******* compute direction ********
  Theta= pbip_get_Theta(x,z,s,w,q,activelnk);
  if (it==0)  dy=[]; end % to call pbip_Newton_direction without errors
  [dx,dy,dz,dw,this_cgit]= ...
      pbip_Newton_direction(q,N,L,Theta,rb,rc,rxz,rsw,x,s,w,activelnk,dy,type_comp_dy);
  cgit= cgit+this_cgit;

  % **** write info of current point and initial information if it== 0 ***
  if (it == 0) 
    pbip_write_problem_information(N,activelnk,type_comp_dy,q);
  end
 pbip_write_current_point_info(it,mu,normrb,normrc,this_cgit,pobj,dobj,gap);
  
  %******* compute steplengths ******
  [alfa_p,alfa_d]= pbip_steplengths(u,x,z,w,dx,dz,dw,BIG);

  %*** update new point ******
  [x,y,z,w]= pbip_update_newpoint(x,y,z,w,dx,dy,dz,dw,alfa_p,alfa_d,activelnk);
  % detection of inactive linking constraints is done before
  % computing (in pbip_KKT_residuals) the constraints Ax and A'ty;
  % then we can avoid to undo the effect of the slacks of the removed
  % constraints
  if (it > 1)
    activelnk= pbip_update_inactivelnk(gap,x,u,y,w,activelnk,c,q);
  end
  it= it+1;
  s= pbip_get_s(u,x,activelnk);
  [rb,rc,rxz,rsw,normrb,normrc,mu] = ...
      pbip_KKT_residuals(x,y,z,s,w,activelnk,b,c,q,normb,normc,N,L);  
  prevgap= gap;
  [pobj,dobj,gap]= pbip_objective_functions(b,c,q,u,x,y,w,activelnk);
  if (type_comp_dy==1 || type_comp_dy >=4) & (gap < 5.0e-1) & (gap > 1.05*prevgap)
    type_comp_dy=2;
    fprintf('switching to full Cholesky direction\n');
  end
end
pbip_write_current_point_info(it,mu,normrb,normrc,0,pobj,dobj,gap);
[x,y,w,z]= pbip_postprocess_inactive(x,y,w,z,b,L,N,activelnk);
cpu_total= cputime-cpu_total;

if (orig_type_comp_dy < 3) 
    fprintf('CPU total  cpu symb. fact.   cpu num. fact.    cpu sol. fact.    cpu total fact\n');
    fprintf('%-15.5f   %-15.5f   %-15.5f   %-15.5f   %-15.5f\n',...
        cpu_total,cpu_symfct,cpu_numfct,cpu_solfct,cpu_symfct+cpu_numfct+cpu_solfct);
elseif (orig_type_comp_dy == 3)
    fprintf('CPU total         cpu min-deg-ord    cpu cholinc       cpu precond-pcg  cpu total precond\n');
    fprintf('%-15.5f   %-15.5f    %-15.5f   %-15.5f  %-15.5f\n',...
        cpu_total,cpu_mindegreeord,cpu_cholinc,cpu_pcg_cholinc,cpu_mindegreeord+cpu_cholinc+cpu_pcg_cholinc);
elseif (orig_type_comp_dy > 3)
    fprintf('CPU total  cpu symb. fact.   cpu num. fact.    cpu sol. fact.    cpu total fact\n');
    fprintf('%-15.5f   %-15.5f   %-15.5f   %-15.5f   %-15.5f\n',...
        cpu_total,cpu_symfct,cpu_numfct,cpu_solfct,cpu_symfct+cpu_numfct+cpu_solfct);

    fprintf('           cpu min-deg-ord    cpu cholinc       cpu precond-pcg  cpu total cholinc\n');
    fprintf('                  %-15.5f   %-15.5f   %-15.5f  %-15.5f\n',...
        cpu_mindegreeord,cpu_cholinc,cpu_pcg_cholinc,cpu_mindegreeord+cpu_cholinc+cpu_pcg_cholinc);
    fprintf('CPU total fact and cholinc\n');
    fprintf('%-15.5f\n',cpu_symfct+cpu_numfct+cpu_solfct+cpu_mindegreeord+cpu_cholinc+cpu_pcg_cholinc)
end        



function [x,y,w,z]= pbip_postprocess_inactive(x,y,w,z,b,L,N,activelnk);
global m n k l km kn;
global set_with_ub set_without_ub;
% compute slacks of inactive constraints, and its dual variables y,w,z
% y(inactive), w(inactive) and z(inactive) were  set to 0

% compute set of inactive
aux(1:l)= 1;
aux(activelnk)= 0;
inactivelnk= find(aux)';

% x,y,w,z
%
x(inactivelnk+kn)= 0;
Ax= pbip_Ax(N,L,x,activelnk);
x(inactivelnk+kn)= b(inactivelnk+km)-Ax(inactivelnk+km);
%
y(inactivelnk+km)= 0;
%
w(inactivelnk+kn)= 0; % must this be done?
z(inactivelnk+kn)= 0; % must this be done?
w(set_without_ub)= 0;

function [dx,dy,dz,dw,this_cgit]= ...
     pbip_Newton_direction(q,N,L,Theta,rb,rc,rxz,rsw,x,s,w,activelnk,dy,type_comp_dy)
global pcgtol;
global m n k l km kn;
global set_with_ub set_without_ub;
%
% Computes the standard Newton direction solving system with A*Theta*A'  
% by Cholesky factorizations of the k diagonal blocks and the Jordi Castro's
% power series preconditioner for the Schur complement system
%

% vector r= rc + S^{-1}*rsw - X^{-1}*rxz: needed for dy and dx
r= rc(1:kn) + rsw(1:kn) ./ s(1:kn) - rxz(1:kn) ./ x(1:kn);
r(kn+1:kn+l)= 0;
r(activelnk+kn)= rc(activelnk+kn) + rsw(activelnk+kn) ./ s(activelnk+kn) - rxz(activelnk+kn) ./ x(activelnk+kn);

% ***** dy *****
% First compute rhs term  of (A*Theta*A')dy= rb+A*Theta*r

% rhs of system for dy rhsdy= rb+A*Theta*r
% we can add the vectors since components of rb and A*Theta*r associated
% to inactive linking constraints were set to 0 when computing the
% KKT residuals and by pbip_Ax(). Moreover, the components of 
% Theta(activelnk,k+1) were set to 0, then we can perform a simple 
% product Theta*r.
rhsdy= rb+pbip_Ax(N,L,Theta .* r,activelnk);

if (type_comp_dy == 1) 
  % Cholesky for blocks, PCG for linking constraints using power
  % series preconditioner
  [dy this_cgit]= pbip_compute_dy_cholpcg(rhsdy,N,L,Theta,activelnk,dy);
elseif (type_comp_dy == 2) 
  % Cholesky for full system 
  [dy]= pbip_compute_dy_fullchol(rhsdy,Theta,activelnk,N,L);
  this_cgit= 0;
elseif (type_comp_dy == 3)
  % PCG with IC of Normal Equations
  [dy this_cgit]= pbip_compute_dy_cholinc_pcg(rhsdy,Theta,activelnk,N,L,dy);
elseif (type_comp_dy == 4)
  % Cholesky for blocks, PCG for linking constraints using IC of D
  % (D is submatrix of Schur complement S= D -C'*B^{-1}C of A*Theta*A)
  [dy this_cgit]= pbip_compute_dy_cholpcg_ic_d(rhsdy,N,L,Theta,activelnk,dy);
elseif (type_comp_dy == 5)
  % Cholesky for blocks, PCG for linking constraints using IC of 
  % D-C' diag(B)^{-1}C (Schur complement of A*Theta*A is S= D -C'*B^{-1}C)
  [dy this_cgit]= pbip_compute_dy_cholpcg_ic_dcbc(rhsdy,N,L,Theta,activelnk,dy);
  else
  % no other choice for computing dy up to now
end

% ***** dx *****
% components of inactive constraints were set to 0 in r, Theta and pbip_Aty()
Atdy= pbip_Aty(N,L,dy,activelnk); % used later for dz 
dx= Theta .* (Atdy - r);

% ***** dw *****
% components of inactive constraints were set to 0 in s,rsw,and dx
% so we must avoid division by 0 for s() terms
dw= (rsw(1:kn) + w(1:kn) .* dx(1:kn)) ./ s(1:kn);
dw(kn+1:kn+l)= 0;
dw(activelnk+kn)= (rsw(activelnk+kn) + w(activelnk+kn) .* dx(activelnk+kn)) ./ s(activelnk+kn);
dw(set_without_ub)= 0;

% ***** dz *****
% components of inactive constraints were set to 0 in rc,dw,dx and pbip_Aty()
% Atdy already computed for dx
dz= rc + dw + q .* dx - Atdy;



function [dy]= pbip_compute_dy_fullchol(rhsdy,Theta,activelnk,N,L)
global spchoA;
global made_symbfct_AA;
global A;
global m n k l km kn;
global it;
%
% Solves (A*Theta*A')dy = rhsdy by Cholesky of full system
% Note that system (A*Theta*A') is really of dimension km+length(activelnk),
% but we reuse the initial symbolic factorization,
% where the dimension of the system was km+l; 
% then we need to use a row of the identity matrix for the rows
% of inactive constraints to have a nonsingular system without affecting
% the value of remaining dy() components
% Input parameters
% - rhsdy[km+l]: rhs of (A*Theta*A')dy = rhsdy system
% - A[km+l,kn+l]: full constraints matrix
% - Theta[kn+l]: diagonal Theta matrix 
% - activelnk[]: vector of indices of active constraints
% Returns
% - dy(km+l): direction for dual variables y; positions on inactive linking
%   constraints set to 0
if (~made_symbfct_AA) 
  % create full A with all l linking constraints
  A= pbip_get_A(N,L);
  % perform initial symbolic factorization of A*A'
  % use spones to avoid cancelations forming A*A'; matlab reduces in
  % this case the number of nonzeros, which do not match the number
  % of A*Theta*A' in future iterations (Theta avoids cancelations).
  % scilab does not reduce the size if cancelations appear
  spchoA= pbip_symbfct(spones(A)*spones(A'));
  made_symbfct_AA= logical(1);
end
% Components of inactive constraints are 0 in Theta.
% Rows/columns of inactive linking constraints of A*TheTa*A' 
% set to 0. The diagonal can be set to 0 (Ng-Peyton sets a 
% large pivot in this case) or to 1 (as did here).
% In both cases we avoid singularity without perturbing the solution
AThetaAt= A*diag(sparse(Theta))*A';
aux= (1:l)~=0; % vector of l's true booleans
aux(activelnk)= logical(0); % set false in positions of active constraints
inactivelnk= find(aux); % positions of inactive constraints
AThetaAt(km+inactivelnk,:)= 0;
AThetaAt(:,km+inactivelnk)= 0;
AThetaAt(km+inactivelnk,km+inactivelnk)= speye(length(inactivelnk),length(inactivelnk));

spchoA= pbip_numfct(AThetaAt,spchoA);

% components of inactive constraints are 0 in rhsdy, so associated dy()
% components will be 0
dy= pbip_solfct(rhsdy,spchoA);


function [dy,this_cgit]= pbip_compute_dy_cholinc_pcg(rhsdy,Theta,activelnk,N,L,dy)
global made_perm_AA;
global perm_AA;
global pcgtol;
global cpu_my_pcg_precond;
global cpu_mindegreeord cpu_cholinc cpu_pcg_cholinc;
global droptol_cholinc switch_droptol;
global A;
global m n k l km kn;
global it;
%
% Solves (A*Theta*A')dy = rhsdy by PCG of full system using incomplete 
% Cholesky as preconditioner of full system
% Note that system (A*Theta*A') is really of dimension km+length(activelnk),
% but we reuse the initial symbolic factorization,
% where the dimension of the system was km+l; 
% then we need to use a row of the identity matrix for the rows
% of inactive constraints to have a nonsingular system without affecting
% the value of remaining dy() components
% Input parameters
% - rhsdy[km+l]: rhs of (A*Theta*A')dy = rhsdy system
% - A[km+l,kn+l]: full constraints matrix
% - Theta[kn+l]: diagonal Theta matrix 
% - activelnk[]: vector of indices of active constraints
% - dy[km+l]: dy of previous iteration, used for initial solution of PCG
% Returns
% - dy(km+l): direction for dual variables y; positions on inactive linking
%   constraints set to 0
if (~made_perm_AA) 
  % create full A with all l linking constraints
  A= pbip_get_A(N,L);
  % get initial permutation of A*A'
  % use spones to avoid cancelations forming A*A'; matlab reduces in
  % this case the number of nonzeros, which do not match the number
  % of A*Theta*A' in future iterations (Theta avoids cancelations).
  % scilab does not reduce the size if cancelations appear
  t= cputime;
  % two ways of computing perm_AA; uncommented one of them
  % this is the first way of obtaining perm_AA using Matlab routines
  %%%%% perm_AA= colamd(spones(A'));
  % this is the second way, using ordmdd of Sparspak/Ng-Peyton
  Maux= spones(A)*spones(A');
  Mauxdiag = diag(diag(Maux)); 
  Maux = Maux - Mauxdiag;
  if (nnz(Maux)>0) %Maux is non-diagonal
    %min-degree ordering  
    [perm_AA,invp]=ordmmd(Maux);
      else % Maux is diagonal; use the original ordering
    perm_AA= (1:size(Maux,1))';
  end
  %
  cpu_mindegreeord= cpu_mindegreeord + (cputime - t);
  made_perm_AA= logical(1);
end
% Components of inactive constraints are 0 in Theta.
% Rows/columns of inactive linking constraints of A*TheTa*A' 
% set to 0 in off-diagonal terms and 1 in diagonal,
% to avoid singularity without perturbing the solution
AThetaAt= A*diag(sparse(Theta))*A';
aux= (1:l)~=0; % vector of l's true booleans
aux(activelnk)= logical(0); % set false in positions of active constraints
inactivelnk= find(aux); % positions of inactive constraints
AThetaAt(km+inactivelnk,:)= 0;
AThetaAt(:,km+inactivelnk)= 0;
AThetaAt(km+inactivelnk,km+inactivelnk)= speye(length(inactivelnk),length(inactivelnk));

% components of inactive constraints are 0 in rhsdy, so associated dy()
% components will be 0
t= cputime;
% incomplete Cholesky factorization, first with 0 fill in
if (~switch_droptol)
    [R,iexit]= cholinc(AThetaAt(perm_AA,perm_AA),'0');
    % if iexit~=0 R incomplete Cholesky with 0 fill-in not possible
    % then we use a drop-tolerance incomplete Cholesky
    if (iexit > 0)
        fprintf('Incomplete Cholesky with 0 fill-in does not exist\n');
        fprintf('Switching to drop-tolerance fill-in\n');
        switch_droptol= logical(1);
    end
end
if (switch_droptol)
    R= cholinc(AThetaAt(perm_AA,perm_AA),droptol_cholinc);
    droptol_cholinc= droptol_cholinc*0.8;
end
cpu_cholinc= cpu_cholinc + (cputime-t);
% PCG
if (it==0) 
% first iteration, use 0 as starting point
  sol= zeros(km+l,1);
else
% other iterations: use as starting point the solution of previous iteration
  sol= dy(perm_AA);
end
pcgtol= max(pcgtol*0.95,1.0e-6);
pcgmaxit= km+l;
[dy(perm_AA,1),flag,err,this_cgit]= my_pcg(AThetaAt(perm_AA,perm_AA),rhsdy(perm_AA),pcgtol,pcgmaxit,{R',R},sol);
cpu_pcg_cholinc= cpu_pcg_cholinc + cpu_my_pcg_precond;
if (flag>0) & (err > 10*pcgtol) 
     error('PCG did not converge');
end

function [A]= pbip_get_A(N,L)
global m n k l km kn;
global ini_m ini_n;
%
% Creates the full system matrix provided the block constraints N,
% and linking constraints L.
%
A= sparse(km+l,kn+l);
if (issparse(N)) % N is a single matrix
  if (issparse(L)) % L is single matrix
    for i=1:k
      A((i-1)*m+1:i*m,(i-1)*n+1:i*n)= N; % diagonal blocks
      A(km+1:km+l,(i-1)*n+1:i*n)= L; % linking costraints 
    end
  else % L is a list of matrices
    for i=1:k
      A((i-1)*m+1:i*m,(i-1)*n+1:i*n)= N; % diagonal blocks
      A(km+1:km+l,(i-1)*n+1:i*n)= L{i}; % linking costraints     
    end
  end
else % N is a list of matrices
  if (issparse(L)) % L is single matrix
    for i=1:k
      A(ini_m(i):ini_m(i+1)-1,ini_n(i):ini_n(i+1)-1)= N{i}; % diagonal blocks
      A(km+1:km+l,ini_n(i):ini_n(i+1)-1)= L; % linking costraints 
    end    
  else % L is a list of matrices
    for i=1:k
      A(ini_m(i):ini_m(i+1)-1,ini_n(i):ini_n(i+1)-1)= N{i}; % diagonal blocks
      A(km+1:km+l,ini_n(i):ini_n(i+1)-1)= L{i}; % linking costraints 
    end    
  end
end
A(km+1:km+l,kn+1:kn+l)= speye(l,l); % slacks of linking constraints


function [dy,this_cgit]= pbip_compute_dy_cholpcg(rhsdy,N,L,Theta,activelnk,dy)
global spchoN all_lnzN spchoD;
global made_symbfct_NN made_symbfct_D;
global pcgtol;
global m n k l km kn;
global ini_m ini_n;
global it;
%
% Solves (A*Theta*A')dy = rhsdy by Choleskdy for blocks and PCG for linking
% constraints
% Note that system (A*Theta*A') is really of dimension km+length(activelnk), 
% then we need to compact the part of linking constraints when 
% applying the PCG, considering only the active ones
% Input parameters
% - N[m,n]: block constraints matrix
% - L[l,n]: linking constraints matrix
% - Theta[kn+l]: diagonal Theta matrix 
% - activelnk[]: vector of indices of active constraints
% - dy[km+l]: dy of previous iteration, used for initial solution of PCG
% - rhsdy[km+l]: rhs of (A*Theta*A')dy = rhsdy system
% Returns
% - dy(km+l): direction for dual variables y; positions on inactive linking
%   constraints set to 0
% - this_cgit: number of PCG iterations for this IP iteration

if (~made_symbfct_NN) 
  % perform initial symbolic factorization of N*N'
  % use spones to avoid cancelations forming A*A'; matlab reduces in
  % this case the number of nonzeros, which do not match the number
  % of A*Theta*A' in future iterations (Theta avoids cancelations).
  % scilab does not reduce the size if cancelations appear
  if (issparse(N)) % N is a single matrix
    spchoN= pbip_symbfct(spones(N)*spones(N'));
  else % N is a list of matrices
    % spchoN set as list of factorization structures
    for i=1:k
      spchoN{i}= pbip_symbfct(spones(N{i})*spones(N{i}')); 
    end
  end
  made_symbfct_NN= logical(1);
end

% create and factorize the k diagonal blocks of 
% B= [N*Theta_i*N' i= 1..k], wich appear when computing (A*Theta*A')
% Store the factorizations in all_lnzN or spchoN depending on 
% N is a single matrix or list of matrices. In the first case
% the symbolic factorization is common for all blocks
if (issparse(N)) % N is a single matrix
  for i=1:k
    spchoN= pbip_numfct(N*diag(sparse(Theta((i-1)*n+1:i*n)))*N',spchoN);
    all_lnzN(1:spchoN{2},i)= spchoN{11};
  end
else % N is a list of matrices
  for i=1:k
    spchoN{i}= pbip_numfct(N{i}*diag(sparse(Theta(ini_n(i):ini_n(i+1)-1)))*N{i}',spchoN{i});
  end 
end

% particular case of no linking constraints: only block constraints components
% solved 
if (length(activelnk) == 0) 
  dy= pbip_solve_NThetaNt(rhsdy);
  this_cgit= 0;
  return;
end

% compute dy2 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving (D-C'*B^(-1)*C) dy2 = (rhsdy2-C'*B^(-1)*rhsdy1)
% by a PCG method
% (D-C'*B^(-1)*C) is the Schur complement of A*Theta*A'
% 
% compute bdy2= (rhsdy2-C'*B^(-1)*rhsdy1)
rhsdy1= rhsdy(1:km); % rhsdy1 is used later for computing dy1
vaux= pbip_solve_NThetaNt(rhsdy1);
vaux2= rhsdy(km+1:km+l)-pbip_Ctv(vaux,N,L,Theta,activelnk);
bdy2= vaux2(activelnk); % select only active constraints rows
%
% compute D= sum{i=1:k} L*Theta_i*L' + diag(Theta_{k+1}), 
% only for active linking constraints.
% D is the first matrix of the Schur complement (D-C'*B^(-1)*C)
D= pbip_get_D(L,Theta,activelnk,N);
if (~made_symbfct_D) 
  % perform first and unique symbolic factorization; it will be used
  % for all the iterations, even if constraints are removed
  % D is built such that in contains a row of the identity matrix for
  % removed constraints, such that the symbolic factorization can be reused
  % and that the factors of removed constraints do not affect the solution
  % of current active constraints
  spchoD= pbip_symbfct(D);
  made_symbfct_D= logical(1);
end
spchoD= pbip_numfct(D,spchoD);
% set tolerance and maximum iterations for PCG
pcgtol= max(pcgtol*0.95,1.0e-8);
pcgmaxit= 2*size(bdy2,1);
% PCG
if (it==0) 
% first iteration, use 0 as starting point
  sol= zeros(size(bdy2,1),1);
else
% other iterations: use as starting point the solution of previous iterations
  sol= dy(activelnk+km);
end
% * pbip_PCG_Hv is a function that evaluates the matrix system, providing
% the product H*v=(D-C'*B^(-1)*C)v
% * pbip_PCH_Hzr is a function that evaluates the preconditioner, 
% approximately solving z= ~H\r, where H= D-C'*B^(-1)*C, and its inverse
% is obtained as an infinity power series expansion; considering the
% first term of the expansion, we approximate z= H\r, by z= ~H\r
[sol,flag,err,this_cgit,res]= ...
    my_pcg(@pbip_PCG_Hv,bdy2,pcgtol,pcgmaxit,@pbip_PCG_Hzr,sol,...
            l,N,L,Theta,activelnk,D);
% check result of PCG
% only if no convergence and err (relative norm of residual) is large
% (e.g., 10 times the required tolerance)
if (flag>0) & (err > 10*pcgtol) 
     error('PCG did not converge');
end
dy2= zeros(l,1);
dy2(activelnk)= sol;

% compute dy1 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving B dy1= rhsdy1 - C*dy2
dy1= pbip_solve_NThetaNt(rhsdy1-pbip_Cv(dy2,N,L,Theta,activelnk));

%building dy
dy=[dy1;dy2];


function [dy,this_cgit]= pbip_compute_dy_cholpcg_ic_d(rhsdy,N,L,Theta,activelnk,dy)
global spchoN all_lnzN;
global made_symbfct_NN;
global made_perm_D;
global perm_D;
global RD;
global cpu_mindegreeord cpu_cholinc cpu_pcg_cholinc;
global droptol_cholinc switch_droptol;
global cpu_my_pcg_precond;
global pcgtol;
global m n k l km kn;
global ini_m ini_n;
global it;
%
% Solves (A*Theta*A')dy = rhsdy by Choleskdy for blocks and PCG for linking
% constraints using IC of D as preconditioner (instead of power series
% preconditioner).
% Note that system (A*Theta*A') is really of dimension km+length(activelnk), 
% then we need to compact the part of linking constraints when 
% applying the PCG, considering only the active ones
% Input parameters
% - N[m,n]: block constraints matrix
% - L[l,n]: linking constraints matrix
% - Theta[kn+l]: diagonal Theta matrix 
% - activelnk[]: vector of indices of active constraints
% - dy[km+l]: dy of previous iteration, used for initial solution of PCG
% - rhsdy[km+l]: rhs of (A*Theta*A')dy = rhsdy system
% Returns
% - dy(km+l): direction for dual variables y; positions on inactive linking
%   constraints set to 0
% - this_cgit: number of PCG iterations for this IP iteration

if (~made_symbfct_NN) 
  % perform initial symbolic factorization of N*N'
  % use spones to avoid cancelations forming A*A'; matlab reduces in
  % this case the number of nonzeros, which do not match the number
  % of A*Theta*A' in future iterations (Theta avoids cancelations).
  % scilab does not reduce the size if cancelations appear
  if (issparse(N)) % N is a single matrix
    spchoN= pbip_symbfct(spones(N)*spones(N'));
  else % N is a list of matrices
    % spchoN set as list of factorization structures
    for i=1:k
      spchoN{i}= pbip_symbfct(spones(N{i})*spones(N{i}')); 
    end
  end
  made_symbfct_NN= logical(1);
end

% create and factorize the k diagonal blocks of 
% B= [N*Theta_i*N' i= 1..k], wich appear when computing (A*Theta*A')
% Store the factorizations in all_lnzN or spchoN depending on 
% N is a single matrix or list of matrices. In the first case
% the symbolic factorization is common for all blocks
if (issparse(N)) % N is a single matrix
  for i=1:k
    spchoN= pbip_numfct(N*diag(sparse(Theta((i-1)*n+1:i*n)))*N',spchoN);
    all_lnzN(1:spchoN{2},i)= spchoN{11};
  end
else % N is a list of matrices
  for i=1:k
    spchoN{i}= pbip_numfct(N{i}*diag(sparse(Theta(ini_n(i):ini_n(i+1)-1)))*N{i}',spchoN{i});
  end 
end

% particular case of no linking constraints: only block constraints components
% solved 
if (length(activelnk) == 0) 
  dy= pbip_solve_NThetaNt(rhsdy);
  this_cgit= 0;
  return;
end

% compute dy2 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving (D-C'*B^(-1)*C) dy2 = (rhsdy2-C'*B^(-1)*rhsdy1)
% by a PCG method and IC of D as preconditioner
% (D-C'*B^(-1)*C) is the Schur complement of A*Theta*A'
% 
% compute bdy2= (rhsdy2-C'*B^(-1)*rhsdy1)
rhsdy1= rhsdy(1:km); % rhsdy1 is used later for computing dy1
vaux= pbip_solve_NThetaNt(rhsdy1);
vaux2= rhsdy(km+1:km+l)-pbip_Ctv(vaux,N,L,Theta,activelnk);
bdy2= vaux2(activelnk); % select only active constraints rows
%
% compute D= sum{i=1:k} L*Theta_i*L' + diag(Theta_{k+1}), 
% only for active linking constraints.
% D is the first matrix of the Schur complement (D-C'*B^(-1)*C)
D= pbip_get_D(L,Theta,activelnk,N);
if (~made_perm_D) 
  % perform first and unique permutation of D; it will be used
  % for all the iterations, even if constraints are removed
  % D is built such that in contains a row of the identity matrix for
  % removed constraints, such that the permutation can be reused
  % and that the factors of removed constraints do not affect the solution
  % of current active constraints
  t= cputime;
  Maux= spones(D);
  Mauxdiag = diag(diag(Maux)); 
  Maux = Maux - Mauxdiag;
  if (nnz(Maux)>0) %Maux is non-diagonal
    %min-degree ordering  
    [perm_D,invp]=ordmmd(Maux);
      else % Maux is diagonal; use the original ordering
    perm_D= (1:size(Maux,1))';
  end
  cpu_mindegreeord= cpu_mindegreeord + (cputime - t);
  made_perm_D= logical(1);
end
t= cputime;
% incomplete Cholesky factorization, first with 0 fill in
if (~switch_droptol)
    [RD,iexit]= cholinc(D(perm_D,perm_D),'0');
    % if iexit~=0 R incomplete Cholesky with 0 fill-in not possible
    % then we use a drop-tolerance incomplete Cholesky
    if (iexit > 0)
        fprintf('Incomplete Cholesky with 0 fill-in does not exist\n');
        fprintf('Switching to drop-tolerance fill-in\n');
        switch_droptol= logical(1);
    end
end
if (switch_droptol)
    RD= cholinc(D(perm_D,perm_D),droptol_cholinc);
    droptol_cholinc= droptol_cholinc*0.8;
end
cpu_cholinc= cpu_cholinc + (cputime-t);
% set tolerance and maximum iterations for PCG
pcgtol= max(pcgtol*0.95,1.0e-8);
pcgmaxit= 2*size(bdy2,1);
% PCG
if (it==0) 
% first iteration, use 0 as starting point
  sol= zeros(size(bdy2,1),1);
else
% other iterations: use as starting point the solution of previous iterations
  sol= dy(activelnk+km);
end
% * pbip_PCG_Hv is a function that evaluates the matrix system, providing
% the product H*v=(D-C'*B^(-1)*C)v
% * pbip_PCH_Hzr is a function that evaluates the preconditioner, 
% approximately solving z= ~H\r, where H= D-C'*B^(-1)*C, and its inverse
% is obtained as an infinity power series expansion; considering the
% first term of the expansion, we approximate z= H\r, by z= ~H\r
[sol,flag,err,this_cgit,res]= ...
    my_pcg(@pbip_PCG_Hv,bdy2,pcgtol,pcgmaxit,@pbip_PCG_Hzr_ICD,sol,...
            l,N,L,Theta,activelnk,D);
cpu_pcg_cholinc= cpu_pcg_cholinc + cpu_my_pcg_precond;
% check result of PCG
% only if no convergence and err (relative norm of residual) is large
% (e.g., 10 times the required tolerance)
if (flag>0) & (err > 10*pcgtol) 
     error('PCG did not converge');
end
dy2= zeros(l,1);
dy2(activelnk)= sol;

% compute dy1 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving B dy1= rhsdy1 - C*dy2
dy1= pbip_solve_NThetaNt(rhsdy1-pbip_Cv(dy2,N,L,Theta,activelnk));

%building dy
dy=[dy1;dy2];

function [dy,this_cgit]= pbip_compute_dy_cholpcg_ic_dcbc(rhsdy,N,L,Theta,activelnk,dy)
global spchoN all_lnzN;
global made_symbfct_NN;
global made_perm_DCBC;
global perm_DCBC;
global RDCBC;
global cpu_mindegreeord cpu_cholinc cpu_pcg_cholinc;
global droptol_cholinc switch_droptol;
global cpu_my_pcg_precond;
global pcgtol;
global m n k l km kn;
global ini_m ini_n;
global it;
%
% Solves (A*Theta*A')dy = rhsdy by Choleskdy for blocks and PCG for linking
% constraints using IC of D-C'diag(B)^{-1}C as preconditioner 
%(instead of power series preconditioner).
% Note that system (A*Theta*A') is really of dimension km+length(activelnk), 
% then we need to compact the part of linking constraints when 
% applying the PCG, considering only the active ones
% Input parameters
% - N[m,n]: block constraints matrix
% - L[l,n]: linking constraints matrix
% - Theta[kn+l]: diagonal Theta matrix 
% - activelnk[]: vector of indices of active constraints
% - dy[km+l]: dy of previous iteration, used for initial solution of PCG
% - rhsdy[km+l]: rhs of (A*Theta*A')dy = rhsdy system
% Returns
% - dy(km+l): direction for dual variables y; positions on inactive linking
%   constraints set to 0
% - this_cgit: number of PCG iterations for this IP iteration

if (~made_symbfct_NN) 
  % perform initial symbolic factorization of N*N'
  % use spones to avoid cancelations forming A*A'; matlab reduces in
  % this case the number of nonzeros, which do not match the number
  % of A*Theta*A' in future iterations (Theta avoids cancelations).
  % scilab does not reduce the size if cancelations appear
  if (issparse(N)) % N is a single matrix
    spchoN= pbip_symbfct(spones(N)*spones(N'));
  else % N is a list of matrices
    % spchoN set as list of factorization structures
    for i=1:k
      spchoN{i}= pbip_symbfct(spones(N{i})*spones(N{i}')); 
    end
  end
  made_symbfct_NN= logical(1);
end

% create and factorize the k diagonal blocks of 
% B= [N*Theta_i*N' i= 1..k], wich appear when computing (A*Theta*A')
% Store the factorizations in all_lnzN or spchoN depending on 
% N is a single matrix or list of matrices. In the first case
% the symbolic factorization is common for all blocks
% Also store diag of B to be used later for the preconditioner
if (issparse(N)) % N is a single matrix
  for i=1:k
    Bi= N*diag(sparse(Theta((i-1)*n+1:i*n)))*N';
    spchoN= pbip_numfct(Bi,spchoN);
    all_lnzN(1:spchoN{2},i)= spchoN{11};
    invdiagB{i}= (1./diag(Bi));
  end
else % N is a list of matrices
  for i=1:k
    Bi= N{i}*diag(sparse(Theta(ini_n(i):ini_n(i+1)-1)))*N{i}';
    spchoN{i}= pbip_numfct(Bi,spchoN{i});
    invdiagB{i}= (1./diag(Bi));
  end 
end

% particular case of no linking constraints: only block constraints components
% solved 
if (length(activelnk) == 0) 
  dy= pbip_solve_NThetaNt(rhsdy);
  this_cgit= 0;
  return;
end

% compute dy2 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving (D-C'*B^(-1)*C) dy2 = (rhsdy2-C'*B^(-1)*rhsdy1)
% by a PCG method and IC of D as preconditioner
% (D-C'*B^(-1)*C) is the Schur complement of A*Theta*A'
% 
% compute bdy2= (rhsdy2-C'*B^(-1)*rhsdy1)
rhsdy1= rhsdy(1:km); % rhsdy1 is used later for computing dy1
vaux= pbip_solve_NThetaNt(rhsdy1);
vaux2= rhsdy(km+1:km+l)-pbip_Ctv(vaux,N,L,Theta,activelnk);
bdy2= vaux2(activelnk); % select only active constraints rows
%
% compute D= sum{i=1:k} L*Theta_i*L' + diag(Theta_{k+1}), 
% only for active linking constraints.
% D is the first matrix of the Schur complement (D-C'*B^(-1)*C)
D= pbip_get_D(L,Theta,activelnk,N);
% compute DCBC= D-C'diag(B)^{-1}C for linking constraints
DCBC= D;
for i=1:k
    % Cti stores C'_i i=1..k; Cti is of dimension l x m_i, but only
    % rows-columns of active linking constraints are computed
    if (issparse(L)) % L is a single matrix
        if (issparse(N)) % N is a single matrix
	    Cti= sparse(l,m);
            Cti(activelnk,:)= L(activelnk,:)*diag(sparse(Theta((i-1)*n+1:i*n))) * N';
        else % N is a list of matrices
	    Cti= sparse(l,m(i));
            Cti(activelnk,:)= L(activelnk,:)*diag(sparse(Theta(ini_n(i):ini_n(i+1)-1))) * N{i}';
        end
    else % L is a list of matrices
        if (issparse(N)) % N is a single matrix
	    Cti= sparse(l,m);
            Cti(activelnk,:)= L{i}(activelnk,:)*diag(sparse(Theta((i-1)*n+1:i*n))) * N';
        else % N is a list of matrices
	    Cti= sparse(l,m(i));
            Cti(activelnk,:)= L{i}(activelnk,:)*diag(sparse(Theta(ini_n(i):ini_n(i+1)-1))) * N{i}';
        end
    end
    CBC= Cti*diag(sparse(invdiagB{i}))*Cti';
    if (~isempty(CBC)) 
      DCBC= DCBC- CBC;
    end
end
if (~made_perm_DCBC) 
  % perform first and unique permutation of D-C' diag(B)^{-1}C; it will be used
  % for all the iterations, even if constraints are removed
  % D is built such that in contains a row of the identity matrix for
  % removed constraints, such that the permutation can be reused
  % and that the factors of removed constraints do not affect the solution
  % of current active constraints
  t= cputime;
  Maux= spones(DCBC);
  Mauxdiag = diag(diag(Maux)); 
  Maux = Maux - Mauxdiag;
  if (nnz(Maux)>0) %Maux is non-diagonal
    %min-degree ordering  
    [perm_DCBC,invp]=ordmmd(Maux);
      else % Maux is diagonal; use the original ordering
    perm_DCBC= (1:size(Maux,1))';
  end
  cpu_mindegreeord= cpu_mindegreeord + (cputime - t);
  made_perm_DCBC= logical(1);
end
t= cputime;
% incomplete Cholesky factorization, first with 0 fill in
if (~switch_droptol)
    [RDCBC,iexit]= cholinc(DCBC(perm_DCBC,perm_DCBC),'0');
    % if iexit~=0 R incomplete Cholesky with 0 fill-in not possible
    % then we use a drop-tolerance incomplete Cholesky
    if (iexit > 0)
        fprintf('Incomplete Cholesky with 0 fill-in does not exist\n');
        fprintf('Switching to drop-tolerance fill-in\n');
        switch_droptol= logical(1);
    end
end
if (switch_droptol)
    RDCBC= cholinc(DCBC(perm_DCBC,perm_DCBC),droptol_cholinc);
    droptol_cholinc= droptol_cholinc*0.8;
end
cpu_cholinc= cpu_cholinc + (cputime-t);
% set tolerance and maximum iterations for PCG
pcgtol= max(pcgtol*0.95,1.0e-8);
pcgmaxit= 2*size(bdy2,1);
% PCG
if (it==0) 
% first iteration, use 0 as starting point
  sol= zeros(size(bdy2,1),1);
else
% other iterations: use as starting point the solution of previous iterations
  sol= dy(activelnk+km);
end
% * pbip_PCG_Hv is a function that evaluates the matrix system, providing
% the product H*v=(D-C'*B^(-1)*C)v
% * pbip_PCH_Hzr is a function that evaluates the preconditioner, 
% approximately solving z= ~H\r, where H= D-C'*B^(-1)*C, and its inverse
% is obtained as an infinity power series expansion; considering the
% first term of the expansion, we approximate z= H\r, by z= ~H\r
[sol,flag,err,this_cgit,res]= ...
    my_pcg(@pbip_PCG_Hv,bdy2,pcgtol,pcgmaxit,@pbip_PCG_Hzr_ICDCBC,sol,...
            l,N,L,Theta,activelnk,D);
cpu_pcg_cholinc= cpu_pcg_cholinc + cpu_my_pcg_precond;
% check result of PCG
% only if no convergence and err (relative norm of residual) is large
% (e.g., 10 times the required tolerance)
if (flag>0) & (err > 10*pcgtol) 
     error('PCG did not converge');
end
dy2= zeros(l,1);
dy2(activelnk)= sol;

% compute dy1 of dy=[dy1, dy2], rhsdy= [rhsdy1, rhsdy2]
% solving B dy1= rhsdy1 - C*dy2
dy1= pbip_solve_NThetaNt(rhsdy1-pbip_Cv(dy2,N,L,Theta,activelnk));

%building dy
dy=[dy1;dy2];

function [D]= pbip_get_D(L,Theta,activelnk,N)
global m n k l km kn;
global ini_m ini_n;
% compute D= sum{i=1:k} L*Theta_i*L' + diag(Theta_{k+1}), 
% only for active linking constraints.
% for inactive constraints set a row of the identity matrix to guarantee
% that numerical factorization can be made when constraints are removed,
% without affecting the solution of active constraints, and avoiding new
% symbolic factorizations.
% D is the first matrix of the Schur complement of A*Theta*A': D-C'*B^(-1)*C
% Returns 
% D[1:l,1:l]: sparse matrix of l rows and l columns, with only rows/columns
% of active constraints filled; the remaining rows/columns have only 1 in
% the diagonal and 0 in off-diagonal
D= ones(l,1);
D(activelnk)= Theta(activelnk+kn);
D= diag(sparse(D));
if (issparse(L))  % L is a single matrix
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      D(activelnk,activelnk)= D(activelnk,activelnk) + ...
	  L(activelnk,:) * diag(sparse(Theta((i-1)*n+1:i*n))) * (L(activelnk,:)');
    end
  else % N is a list of matrices
    for i= 1:k
      D(activelnk,activelnk)= D(activelnk,activelnk) + ...
	  L(activelnk,:) * diag(sparse(Theta(ini_n(i):ini_n(i+1)-1))) * (L(activelnk,:)');
    end
  end
else % L is a list of matrices
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      D(activelnk,activelnk)= D(activelnk,activelnk) + ...
	  L{i}(activelnk,:) * diag(sparse(Theta((i-1)*n+1:i*n))) * (L{i}(activelnk,:)');
    end
  else % N is a list of matrices
    for i= 1:k
      D(activelnk,activelnk)= D(activelnk,activelnk) + ...
	  L{i}(activelnk,:) * diag(sparse(Theta(ini_n(i):ini_n(i+1)-1))) * (L{i}(activelnk,:)');
    end    
  end
end


function [Hv]= pbip_PCG_Hv(v,l,N,L,Theta,activelnk,D)
% Uses N,L,Theta,activelnk,D as global variables
%
% Computes Hv=(D-C'*B^(-1)*C)v
% v has dimension number of active constraints, then we need 
% to use a temporary vector vaux of dimension l (all linking constraints)
vaux= zeros(l,1);
vaux(activelnk)= v;
vaux= D*vaux-pbip_Ctv(pbip_solve_NThetaNt(pbip_Cv(vaux,N,L,Theta,activelnk)),...
        N,L,Theta,activelnk);
Hv= vaux(activelnk); % return only components of active constraints


function [z]= pbip_PCG_Hzr(r,l,N,L,Theta,activelnk)
global m_pw_prec;
global spchoD;
% Uses N,L,Theta,activelnk,D as global variables
%
% Approximate solution of z= H\r, using m_pw_prec terms of the power series
% expansion 
% H^{-1}= [sum{i=0..infinity} (D^{-1}*(C'B^{-1}B)C)^i]*D^{-1}  
% Recommended values are m_pw_prec= 1 or 2, to avoid expensive
% computations
% r and z have dimension number of active constraints, then we need 
% to use temporary vectors raux and zaux of dimension l 
% (all linking constraints)

raux= zeros(l,1);
raux(activelnk)= r;

zaux= pbip_solfct(raux,spchoD);
for i=1:m_pw_prec-1
  zaux= pbip_solfct(pbip_Ctv(pbip_solve_NThetaNt(pbip_Cv(zaux,N,L,Theta,activelnk)),...
        N,L,Theta,activelnk)+raux, spchoD);
end
z= zaux(activelnk); % return only components of active constraints

function [z]= pbip_PCG_Hzr_ICD(r,l,N,L,Theta,activelnk)
global perm_D;
global RD;
global cpu_my_pcg_precond;
% Solves z= (RD'*RD)\r, where RD is IC of D, considering the permutation
% permD
% r and z have dimension number of active constraints, then we need 
% to use temporary vectors raux and zaux of dimension l 
% (all linking constraints)

raux= zeros(l,1);
raux(activelnk)= r;
taux= cputime;
zaux= RD \ (RD' \raux(perm_D));
zaux(perm_D)= zaux;
cpu_my_pcg_precond= cputime-taux;
z= zaux(activelnk); % return only components of active constraints

function [z]= pbip_PCG_Hzr_ICDCBC(r,l,N,L,Theta,activelnk)
global perm_DCBC;
global RDCBC;
global cpu_my_pcg_precond;
% Solves z= (RDCBC'*RDCBC)\r, where RDCBC is IC of D-C'diag(B)^{-1}C, 
% considering the permutation permDCBC
% r and z have dimension number of active constraints, then we need 
% to use temporary vectors raux and zaux of dimension l 
% (all linking constraints)

raux= zeros(l,1);
raux(activelnk)= r;
taux= cputime;
zaux= RDCBC \ (RDCBC' \raux(perm_DCBC));
zaux(perm_DCBC)= zaux;
cpu_my_pcg_precond= cputime-taux;
z= zaux(activelnk); % return only components of active constraints


function [Cv]= pbip_Cv(v,N,L,Theta,activelnk)
global m n k l km kn;
global ini_m ini_n;
%
% Given v[l], this routine computes
% C*v= [N*Theta_i*L'*v i=1..k], using only the columns of N*Theta_i*L' and
% components of v associated to active constraints. C*v is a vector [1:km]
if (issparse(L)) % L is a single matrix
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      Cv(1:m,i)= N*(Theta((i-1)*n+1:i*n) .* ((L(activelnk,:)')*v(activelnk)));
    end
  else % N is a list of matrices
    for i= 1:k
      Cv(ini_m(i):ini_m(i+1)-1,1)= N{i}*(Theta(ini_n(i):ini_n(i+1)-1) .* ((L(activelnk,:)')*v(activelnk)));    
    end
  end
else % L is a list of matrices
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      Cv(1:m,i)= N*(Theta((i-1)*n+1:i*n) .* ((L{i}(activelnk,:)')*v(activelnk)));
    end
  else % N is a list of matrices
    for i= 1:k
      Cv(ini_m(i):ini_m(i+1)-1,1)= N{i}*(Theta(ini_n(i):ini_n(i+1)-1) .* ((L{i}(activelnk,:)')*v(activelnk))); 
    end
  end
end
if (issparse(N)) % N is a single matrix
  Cv= reshape(Cv,km,1);
end
Cv= full(Cv);

function [Ctv]= pbip_Ctv(v,N,L,Theta,activelnk)
global m n k l km kn;
global ini_m ini_n;
%
% Given v[m,k], this routine computes
% Ct*v= sum{i in 1..k} L*Theta_i*N'*v_i, which is a vector of dimension l
% The result vector Ctv only provides values of active linking,
% components of inactive constraints are 0
% constraints

if (issparse(N)) 
  v= reshape(v,m,k);
end
Ctv= zeros(l,1);
if (issparse(L)) % L is a single matrix
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      aux= L(activelnk,:)*(Theta((i-1)*n+1:i*n) .* (N'*v(:,i)));
      Ctv(activelnk)= Ctv(activelnk)+ aux;
    end
  else % N is a list of matrices
    for i= 1:k
      aux= L(activelnk,:)*(Theta(ini_n(i):ini_n(i+1)-1) .* (N{i}'*v(ini_m(i):ini_m(i+1)-1)));
      Ctv(activelnk)= Ctv(activelnk)+ aux;
    end
  end
else % L is a list of matrices
  if (issparse(N)) % N is a single matrix
    for i= 1:k
      aux= L{i}(activelnk,:)*(Theta((i-1)*n+1:i*n) .* (N'*v(:,i)));  
      Ctv(activelnk)= Ctv(activelnk)+ aux;
    end
  else % N is a list of matrices
    for i= 1:k
      aux= L{i}(activelnk,:)*(Theta(ini_n(i):ini_n(i+1)-1) .* (N{i}'*v(ini_m(i):ini_m(i+1)-1)));
      Ctv(activelnk)= Ctv(activelnk)+ aux;
    end
  end
end
Ctv= full(Ctv);


function [sol]= pbip_solve_NThetaNt(v)
global spchoN all_lnzN;
global m n k l km kn;
global ini_m ini_n;
%
% Solves k block systems with N*Theta_i*N' i=1..k
% On input
% - v[m,k]: stores the rhs of the k block systems
% On output
% - sol[m,k]: stores the solution of the k block systems
% detects if we have equal of different blocks by the type of
% the first element of spchoN

if (~iscell(spchoN{1})) % N is a single matrix
  v= reshape(v,m,k);
  for i=1:k
    spchoN{11}= all_lnzN(:,i);
    sol(1:m,i)= pbip_solfct(v(:,i),spchoN);
  end
  sol= reshape(sol,km,1);
else % N is a list of matrices
  for i=1:k
    sol(ini_m(i):ini_m(i+1)-1,1)= pbip_solfct(v(ini_m(i):ini_m(i+1)-1),spchoN{i});
  end  
end

function [alfa_p,alfa_d]= pbip_steplengths(u,x,z,w,dx,dz,dw,BIG)
global RHO;
global m n k l km kn;
%
% Compute the primal and dual steplengths alfa_p and alfa_d such that
% x+alfa_p*dx and z+alfa_d*dz, w+alfa_d*dw are between bounds
% It is assumed that the components of directions dx, dz, dw associated to
% inactive constraints or their slacks were computed as 0 (this is satisfied)
% by pbip_Newton_direction. This simplifies the computation.

EPS= 1.0e-17;

alfa_x= min( min([-x(dx<-EPS) ./ dx(dx<-EPS); BIG]), ...
	min([(u(dx>EPS)-x(dx>EPS)) ./ dx(dx>EPS); BIG]) );

alfa_z= min( min([-z(dz<-EPS) ./ dz(dz<-EPS); BIG]) );
alfa_w= min( min([-w(dw<-EPS) ./ dw(dw<-EPS); BIG]) );

alfa_p= min(RHO*alfa_x,1);
alfa_d= min([RHO*alfa_z,RHO*alfa_w,1]);



function [activelnk]= pbip_update_inactivelnk(gap,x,u,y,w,activelnk,c,q);
global m n k l km kn;
%
% Detect if new linking constraints must be considered inactive
% This is only done if we are close enough to the optimal solution;
% we use (gap < 1.0), where gap was computed before as
%  abs(pobj-dobj)/(1+abs(pobj)).
% Constraint i inactived if its slack x(i) is out of bounds and lagrange 
% multiplier is close to 0:
% (x(i)>>0 and x(i)<<u(i) and y(i)~0) and (c(i)==0 and q(i)==0). 
% The term  (c(i)==0 and q(i)==0) guarantees that slacks appearing in the
% objective function are not removed
% We also avoid inactivation of constraints with small bounds on slacks
% (i.e, u<0.1), since these constraints are always quasi-active

% if not close enough to optimum, or not active constraints
% don't inactive constraints
if (gap > 1.0) | (length(activelnk) == 0) 
  activelnk= activelnk;
  return; 
end

% compute boolean vector inact() of active constraints that will be set
% inactive
inact= (x(activelnk+kn) > 0.1*u(activelnk+kn)) ....
       & (x(activelnk+kn) < 0.9*u(activelnk+kn)) ...
       & (abs(y(activelnk+km)) < 1.0e-2) ...
       & (c(activelnk+kn)== 0) & (q(activelnk+kn)== 0) & (u(activelnk+kn)>0.1);
nremoved= sum(inact);
if (nremoved > 0) 
  fprintf('  Removed Linking Constraints= %d\n',nremoved)
end
activelnk= activelnk(~inact);


function [x,y,z,w]= pbip_update_newpoint(x,y,z,w,dx,dy,dz,dw,alfa_p,alfa_d,activelnk)
global m n k l km kn;
%
% Computes the new iteration point: 
% primal point: x+alfa_p*dx
% dual point: (y,z,w)+alfa_d*(dx,dz,dw)
% Components of y,x,z,w associated to inactive linking constraints 
% are skipped

x(1:kn)= x(1:kn)+alfa_p*dx(1:kn);
x(activelnk+kn)= x(activelnk+kn)+alfa_p*dx(activelnk+kn);

y(1:km)= y(1:km)+alfa_d*dy(1:km);
y(activelnk+km)= y(activelnk+km)+alfa_d*dy(activelnk+km);

z(1:kn)= z(1:kn)+alfa_d*dz(1:kn);
z(activelnk+kn)= z(activelnk+kn)+alfa_d*dz(activelnk+kn);

w(1:kn)= w(1:kn)+alfa_d*dw(1:kn);
w(activelnk+kn)= w(activelnk+kn)+alfa_d*dw(activelnk+kn);



function [Theta]= pbip_get_Theta(x,z,s,w,q,activelnk)
global m n k l km kn;
global set_with_ub set_without_ub;
%
% Computes the diagonal Theta matrix of A*Theta*A'.
% Theta is a kn+l vector.
% Components of Theta() of inactive constraints, set to 0.
%
  Theta= 1 ./( (z(1:kn) ./ x(1:kn)) + (w(1:kn) ./ s(1:kn)) + q(1:kn) );
  Theta(kn+1:kn+l)= 0;
  Theta(activelnk+kn)= 1 ./( (z(activelnk+kn) ./ x(activelnk+kn)) + ...
	(w(activelnk+kn) ./ s(activelnk+kn)) + q(activelnk+kn) );


function [pobj,dobj,gap]= pbip_objective_functions(b,c,q,u,x,y,w,activelnk)
global m n k l km kn;
%
% Computes primal objective pobj= c'x+1/2 x'Qx 
% and dual objective dobj= b'y -u'w-1/2 x'Qx
% Also the relative gap
% Uses that w and u are 0 and a BIG but finite value respectively for
% variables without upper bounds (ie., w*u= 0 if variable has no upper bound)

auxq= 0.5* (sum(q(1:kn).* x(1:kn) .* x(1:kn)) + ...
	    sum(q(activelnk+kn).* (x(activelnk+kn).^2)));

%primal obj.
pobj= sum(c(1:kn).* x(1:kn))+sum(c(activelnk+kn).* x(activelnk+kn));
pobj= pobj+auxq;

%dual obj.
dobj= sum(b(1:km).* y(1:km))+sum(b(activelnk+km).* y(activelnk+km));
dobj= dobj-sum(u(1:kn).* w(1:kn))-sum(u(activelnk+kn).* w(activelnk+kn));
dobj= dobj-auxq;

% 
gap= abs(pobj-dobj)/(1+abs(pobj)); 



function pbip_write_current_point_info(it,mu,normrb,normrc,this_cgit,pobj,dobj,gap)
%
% Prints one line of info about the current point.

fprintf('%-4d %1.2e %14e %1.2e %14e %1.2e  %1.2e  %d \n',...
       it,mu,pobj,normrb,dobj,normrc,gap,this_cgit);


function []= pbip_write_problem_information(N,activelnk,type_comp_dy,q)
global spchoN;
global spchoA;
global A;
global m n k l km kn;
% Write problem dimensions and information of Cholesky factorization
		
fprintf('----------------------------------------------------------------------------------------------\n');
if (issparse(N)) % N is a single matrix
  fprintf('#block variables= %d    #block constraints= %d    blocks= %d   linking constraints= %d\n', n,m,k,l);
else % N is a list of matrices
  fprintf('#all-block variables= %d    #all-block constraints= %d    blocks= %d   linking constraints= %d\n', kn,km,k,l);
end

if (all(q==0)) 
  fprintf('objective is linear\n');
else
  fprintf('objective is quadratic\n');
end
nactive= length(activelnk);
fprintf('constraints= %-15d    variables= %d \n',km+nactive,kn+nactive);
fprintf('linking constraints initially inactive= %d \n',l-nactive);
if (type_comp_dy == 1) 
    fprintf('computation direction dy: power series PCG + Cholesky \n');
    if (issparse(N)) % N is a single matrix
    nnzsubNN= (nnz(spones(N)*spones(N'))-m)/2;
    nnzsubRR= spchoN{2}-m;
    elemsubNN= m*(m-1)/2;
    fprintf('subnonz(N*N)= %-13d subnonz(R*R) = %-15d  fill-in= %4.1f%%\n',...
	    nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(N*N)= %4.1f%%      density(R*R)= %4.1f%%\n',...
	    100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );
  else % N is a list of matrices
    nnzsubNN= 0;
    nnzsubRR= 0;
    elemsubNN= 0;
    for i= 1:k
      % use ceil next for zero elements matrices
      nnzsubNN= nnzsubNN+ceil((nnz(spones(N{i})*spones(N{i}'))-m(i))/2);
      nnzsubRR= nnzsubRR+spchoN{i}{2}-m(i);
      elemsubNN= elemsubNN+m(i)*(m(i)-1)/2;
    end
    fprintf('subnonz(kN*kN)= %-10d subnonz(kR*kR)= %-15d  fill-in= %4.1f%%\n',...
	   nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(kN*kN)= %4.1f%%      density(kR*kR)= %4.1f%%\n',...
	   100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );    
  end    
elseif (type_comp_dy == 2) 
  fprintf('computation direction dy: full Cholesky \n');
  nnzsubAA= (nnz(spones(A)*spones(A'))-(km+l))/2;
  nnzsubRR= spchoA{2}-(km+l);
  elemsubAA= (km+l)*(km+l-1)/2;
  fprintf('subnonz(A*A)= %-13d subnonz(R*R) = %-15d  fill-in= %4.1f%%\n',...
	 nnzsubAA,nnzsubRR,100.0*(nnzsubRR-nnzsubAA)/nnzsubRR);
  fprintf('density(A*A)= %4.1f%%      density(R*R)= %4.1f%%\n',...
	 100.0*nnzsubAA/elemsubAA,100.0*nnzsubRR/elemsubAA );
elseif (type_comp_dy == 3) 
  fprintf('computation direction dy: Incomplete Cholesky PCG of Normal Equations\n');
  nnzsubAA= (nnz(spones(A)*spones(A'))-(km+l))/2;
  elemsubAA= (km+l)*(km+l-1)/2;
  fprintf('subnonz(A*A)= %-13d \n',nnzsubAA);
  fprintf('density(A*A)= %4.1f%% \n',100.0*nnzsubAA/elemsubAA);
elseif (type_comp_dy == 4)
    fprintf('computation direction dy: PCG IC D+ Cholesky \n');
    if (issparse(N)) % N is a single matrix
    nnzsubNN= (nnz(spones(N)*spones(N'))-m)/2;
    nnzsubRR= spchoN{2}-m;
    elemsubNN= m*(m-1)/2;
    fprintf('subnonz(N*N)= %-13d subnonz(R*R) = %-15d  fill-in= %4.1f%%\n',...
	    nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(N*N)= %4.1f%%      density(R*R)= %4.1f%%\n',...
	    100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );
  else % N is a list of matrices
    nnzsubNN= 0;
    nnzsubRR= 0;
    elemsubNN= 0;
    for i= 1:k
      % use ceil next for zero elements matrices
      nnzsubNN= nnzsubNN+ceil((nnz(spones(N{i})*spones(N{i}'))-m(i))/2);
      nnzsubRR= nnzsubRR+spchoN{i}{2}-m(i);
      elemsubNN= elemsubNN+m(i)*(m(i)-1)/2;
    end
    fprintf('subnonz(kN*kN)= %-10d subnonz(kR*kR)= %-15d  fill-in= %4.1f%%\n',...
	   nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(kN*kN)= %4.1f%%      density(kR*kR)= %4.1f%%\n',...
	   100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );    
  end    
elseif (type_comp_dy == 5)
    fprintf('computation direction dy: PCG IC D-Ctdiag(B)^{-1}C + Cholesky \n');
    if (issparse(N)) % N is a single matrix
    nnzsubNN= (nnz(spones(N)*spones(N'))-m)/2;
    nnzsubRR= spchoN{2}-m;
    elemsubNN= m*(m-1)/2;
    fprintf('subnonz(N*N)= %-13d subnonz(R*R) = %-15d  fill-in= %4.1f%%\n',...
	    nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(N*N)= %4.1f%%      density(R*R)= %4.1f%%\n',...
	    100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );
  else % N is a list of matrices
    nnzsubNN= 0;
    nnzsubRR= 0;
    elemsubNN= 0;
    for i= 1:k
      % use ceil next for zero elements matrices
      nnzsubNN= nnzsubNN+ceil((nnz(spones(N{i})*spones(N{i}'))-m(i))/2);
      nnzsubRR= nnzsubRR+spchoN{i}{2}-m(i);
      elemsubNN= elemsubNN+m(i)*(m(i)-1)/2;
    end
    fprintf('subnonz(kN*kN)= %-10d subnonz(kR*kR)= %-15d  fill-in= %4.1f%%\n',...
	   nnzsubNN,nnzsubRR,100.0*(nnzsubRR-nnzsubNN)/nnzsubRR);
    fprintf('density(kN*kN)= %4.1f%%      density(kR*kR)= %4.1f%%\n',...
	   100.0*nnzsubNN/elemsubNN,100.0*nnzsubRR/elemsubNN );    
  end    
else
  % no other choice up to now
end
fprintf('----------------------------------------------------------------------------------------------\n');
fprintf('Iter    mu      Primal Obj.  P. Infeas  Dual Obj.    D. Infeas   Gap     CGit\n');
fprintf('----------------------------------------------------------------------------------------------\n');



function [rb,rc,rxz,rsw,normrb,normrc,mu]= ...
      pbip_KKT_residuals(x,y,z,s,w,activelnk,b,c,q,normb,normc,N,L)
global SIGMA;
global m n k l km kn;
%
% Compute and return the KKT residuals. 
% 
% Output parameters:
%  - rb[km+l]= b-Ax  : primal infeasibility. Only positions of active linking
%    constraints are filled.
%  - rc[kn+l]= c-A'y+w-z+Qx : dual infeasibility. Only positions of slacks of
%    active linking constraints are filled.
%  - rxz[kn+l]= sigma*mu*e- X*z : complemetarity residual for Xz= mu*e, 
%    decreasing mu (see below) by sigma. Positions of inactive linking
%    constraints are not filled 
%  - rsw[kn+l]= sigma*mu*e- S*w : complemetarity residual for Sw= mu*e, 
%    decreasing mu (see below) by sigma. Positions of inactive linking
%    constraints are not filled 
%  - normrb: norm of rc weighted by (1+|c|)
%  - normrb: norm of rb weighted by (1+|b|)
%  - mu= (x'z+s'w)/(kn+length(activelnk)) : centrality parameter

% compute rb
Ax= pbip_Ax(N,L,x,activelnk);
rb= b(1:km)-Ax(1:km);
rb(km+1:km+l,1)= 0;
rb(activelnk+km)= b(activelnk+km)-Ax(activelnk+km);

% compute normrb
normrb= norm(rb)/(1+normb);

% compute rc
Aty= pbip_Aty(N,L,y,activelnk); % get A'*y
% add to -w+z-Q*x to Aty, considering only active linking constraints
% uses that w is 0 for variables with no upper bound
Aty(1:kn)= Aty(1:kn)-w(1:kn)+z(1:kn)-q(1:kn).*x(1:kn);
Aty(activelnk+kn)= Aty(activelnk+kn)-w(activelnk+kn)+...
		       z(activelnk+kn)-q(activelnk+kn).*x(activelnk+kn);

rc= c(1:kn)-Aty(1:kn);
rc(kn+1:kn+l,1)= 0;
rc(activelnk+kn)= c(activelnk+kn)-Aty(activelnk+kn);

% compute normrc
normrc= norm(rc)/(1+normc);

% compute mu
mu= pbip_get_mu(x,z,s,w,activelnk);

% compute rxz
rxz= SIGMA*mu - x(1:kn) .* z(1:kn);
rxz(kn+1:kn+l)= 0;
rxz(activelnk+kn)= SIGMA*mu - x(activelnk+kn) .* z(activelnk+kn);

% compute rsw
% uses that w is 0 for variables without upper bound
rsw= SIGMA*mu-s(1:kn) .* w(1:kn);
rsw(kn+1:kn+l)= 0;
rsw(activelnk+kn)= SIGMA*mu-s(activelnk+kn) .* w(activelnk+kn);



function [Ax]= pbip_Ax(N,L,x,activelnk)
global m n k l km kn;
global ini_m ini_n;
%
% Computes and returns  A*x
% A is the constraints matrix, formed by k diagonal blocks N and last 
% rows with k  L matrices plus the Identity for slacks.
% Computes N*xi for all block i=1..k; and sum{1..k}L*xi + x_{k+1}, for
% the active constraints
% Input parameters:
%  - N: block constraints matrix
%  - L: linking constraints matrix
%  - x[1:kn+l]: primal variables
%  - activelnk: vector of indices of active linking constraints
% Output:
%  - Ax[km+l]: result of A*x. Only values of active linking
%    constraints are filled

% first: block constraints
if (issparse(N)) % N is a single matrix
    % matrix is matlab reshape, v is a [n,k] matrix associated to block variables
    v= reshape(x(1:kn),n,k); 
    Nx= N*v; % N*xi for all i=1..k
    Nx= reshape(Nx,km,1); % Nx is now a km vector
else % N is a list of matrices
    for i=1:k
       Nx(ini_m(i):ini_m(i+1)-1,1)= N{i}*x(ini_n(i):ini_n(i+1)-1);
    end  
end

% second: active linking constraints
Lx= zeros(l,1); 
if (issparse(L)) % L is a single matrix
    if (issparse(N)) % N is a single matrix
        Lx(activelnk)= sum(L(activelnk,:)*v,2)+x(activelnk+kn);
    else % N is a list of matrices
        for i=1:k
            Lx(activelnk)= Lx(activelnk)+L(activelnk,:)*x(ini_n(i):ini_n(i+1)-1);
        end
        Lx(activelnk)= Lx(activelnk) + x(activelnk+kn);
    end  
else % L is a list of matrices
    if (issparse(N)) % N is a single matrix
        for i=1:k
            Lx(activelnk)= Lx(activelnk) + L{i}(activelnk,:)*v(:,i);
        end
        Lx(activelnk)= Lx(activelnk) + x(activelnk+kn);
    else % N is a list of matrices
        for i=1:k
            Lx(activelnk)= Lx(activelnk) + L{i}(activelnk,:)*x(ini_n(i):ini_n(i+1)-1);
        end
        Lx(activelnk)= Lx(activelnk) + x(activelnk+kn);        
    end
end

% join values of block and linking constraints
Ax= [Nx ; Lx];


function [Aty]= pbip_Aty(N,L,y,activelnk)
global m n k l km kn;
global ini_m ini_n;
%
% Computes and returns A'*y 
% A is the constraints matrix, formed by k diagonal blocks N and last 
% rows with k L matrices plus the Identity for slacks.
% Input parameters:
%  - N: block constraints matrix
%  - L: linking constraints matrix
%  - y[km+l],z[kn+l],w[kn+l]: dual variables
%  - activelnk: vector of indices of active linking constraints
% Output:
%  - Aty[kn+l]: result of A'*y. Only values of active 
%    linking constraints are added to each column. 

% effect of dual variables of block constraints  
if (issparse(N)) % N is a single matrix
  v= reshape(y(1:km),m,k); % v contains block dual variables in a [m,k] matrix
  Nty= N'*v; % N'*yi i=1..k
else % N is a list of matrices
  for i=1:k
    Nty(ini_n(i):ini_n(i+1)-1,1)= N{i}'*y(ini_m(i):ini_m(i+1)-1);
  end  
end

% add effects of dual variables of linking constraints
% don't added if Lty=[] (== if length(activelnk)==0), otherwise
% Nty will be set empty!!!!
if (length(activelnk) > 0) 
    if (issparse(L)) %  L is a single matrix
      Lty= (y(activelnk+km)'*L(activelnk,:))'; % L'*ylnk
      if (issparse(N)) % N is a single matrix
        for i=1:k
          Nty(:,i)= Nty(:,i)+Lty; 
        end
      else % N is a list of matrices
        for i=1:k
          Nty(ini_n(i):ini_n(i+1)-1)= Nty(ini_n(i):ini_n(i+1)-1)+Lty; 
        end	
      end
    else % L is a list of matrices
      if (issparse(N)) % N is a single matrix
        for i=1:k
            Nty(:,i)= Nty(:,i)+(y(activelnk+km)'*L{i}(activelnk,:))'; 
        end    
      else % N is a list of matrices
        for i=1:k
            Nty(ini_n(i):ini_n(i+1)-1)= Nty(ini_n(i):ini_n(i+1)-1)+(y(activelnk+km)'*L{i}(activelnk,:))'; 
        end    	
      end
    end
end

if (issparse(N)) % N is a single matrix
% reshape Nty for Aty 
  Aty= reshape(Nty,kn,1);
else % N is a list of matrices
  Aty= Nty;
end

%add Identity part of slacks of active constraints
Aty(kn+1:kn+l)= 0;
Aty(activelnk+kn)= y(activelnk+km);



function [normb,normc]= pbip_normb_normc(b,c,activelnk)
global m n k l km kn;
%
% Compute and return the norm of b and c
% 
% Input parameters:
%  - b[km+l]: right-hand-side 
%  - c[kn+l]: linear costs 
% Output parameters:
%  - normb: L2 norm of b (only active linking constraints)
%  - normc: L2 norm of c (only slacks of active linking constraints)

% add displacement km to have indices of active rows in numbering 1:km+l
normb= sqrt(norm(b(1:km))^2+norm(b(activelnk+km))^2);

% add displacement kn to have indices of active slacks in numbering 1:kn+l
normc= sqrt(norm(c(1:kn))^2+norm(c(activelnk+kn))^2);




function [s]= pbip_get_s(u,x,activelnk)
global m n k l km kn;
global set_with_ub set_without_ub;
%
% Compute and return the slacks of upper bounds of block variables and 
% slacks of linking constraints: x+s= u. 
% Only slacks of slacks of active linking constraints filled
% s of variables with BIG upper bounds computed set to very large value
s= u(1:kn)-x(1:kn);
s(kn+1:kn+l)= 0;
s(activelnk+kn)= u(activelnk+kn)-x(activelnk+kn);
s(set_without_ub)= 1.0e+128;


function [mu]= pbip_get_mu(x,z,s,w,activelnk);
global m n k l km kn;
global set_with_ub set_without_ub;
%
% Compute and returns the centrality at current point, computed as
% mu= (x'z+s'w)/(kn+length(activelnk))
% uses that w and s of variables without upper bound are 0 and a finite (large number)

% Values associated to inactive linking constraints are not computed
mu= sum(x(1:kn) .* z(1:kn)) + sum(s(1:kn) .* w(1:kn));
mu= mu + sum(x(activelnk+kn) .* z(activelnk+kn)) ...
         + sum(s(activelnk+kn) .* w(activelnk+kn));
mu= mu/(kn+length(activelnk)+length(set_with_ub));


function [x,y,z,w]= pbip_initial_point(N,c,q,u,activelnk,initype)
global m n k l km kn;
global ini_m ini_n;
global set_with_ub set_without_ub;
%
% Compute the initial point to start the optimization
%
% Input parameters:
%  - N: node-variable incidence matrix
%  - c: linear costs
%  - q: quadratic costs
%  - u: bounds of variables per block and slacks
%  - activelnk: vector of indcies of active linking constraints
%  - initype: if 1, simple initial point; if 2, Mehrotra-like initialization.
%    see details below 
% Output parameters:
%  - x,y,z,w: initial values of primal (x) and dual (y,z,w) variables
%    y are the multipliers of the equality constraints (block + linking 
%    constraints); z are the multipliers of x>= 0 constraints; w are
%    the multipliers of x<= uk constraints.

x= zeros(kn+l,1);
y= zeros(km+l,1);
z=x;
w=x;

X0= 100.0; % default value far enough from 0

if (initype==1) 
  % Set the following initialization:
  % Primal variables far enough from bounds:
  % x= min(u/2,X0), where u is upper bound and X0 default large value
  % slacks of inactive constraints not filled
  x(1:kn)= min(u(1:kn)/2,X0);
  % avoid problems with min if no active constraint exists
  if (length(activelnk)>0)  
     x(activelnk+kn)= min(u(activelnk+kn)/2,X0);
  end
  % Dual variables satisfying feasibility: A'y-w+z-Qx=c; complementarity 
  % ignored:
  % y= 0
  % -w+z= c+Qx, and w>=0, z>=0
  y(1:km+l)= 0.0; % consider l positions for active-inactive linking consts
  % duals variables of slacks of inactive constraints not filled 
  z(1:kn)= X0;
  w(1:kn)= X0;
  z(activelnk+kn)= X0;
  w(activelnk+kn)= X0;
  aux= c + q.*x;
  z(aux>0) = z(aux>0) + aux(aux>0);
  w(aux<0) = w(aux<0) - aux(aux<0);
  w(set_without_ub)= 0; % w of vars without upper bounds are 0 all over the code
elseif (initype== 2)
  % Mehrotra like initialization, not yet implemented
end


function [spcho]= pbip_symbfct(M)
global cpu_symfct;
%
% Symbolic factorization for sparse Cholesky factorization.
% symbfct(M) does a minimum degree ordering and a symbolic
% factorization for a symmetric square sparse matrix M.
% It calls ordmmd and symfct routines of Ng-Peyton sparse cholesky package
% This routine is derived from lipsol (scilab port)
%
% Returns cell array spcho with the followings variables:
%        - xlnz,nnzl,xsuper,xlindx,lindx,snode,split,tmpsiz,perm,invp:
%          this group of variables is common for any factorization with
%          matrices with same topology
%        - lnz (numerical values of factorization), initially []: this
%          variable is dependent of each particular matrix

t= cputime;
Mdiag = diag(diag(M)); 
M = M - Mdiag;
if (nnz(M)>0)
    %min-degree ordering
    [perm,invp]=ordmmd(M);
    % symbolic factorization, only if M is not diagonal
    cachsz=16;  
    [xlnz,nnzl,xsuper,xlindx,lindx,snode,split,tmpsiz,perm,invp] = ...
                         symfct(M,perm,invp,cachsz);
    spcho= {xlnz,nnzl,xsuper,xlindx,lindx,snode,split,tmpsiz,perm,invp,[]};
else
    spcho= cell(1,11); % empty spcho
    spcho{2}= 0; % 0 nnzl elements, used later for problem statistics
end
cpu_symfct= cpu_symfct+cputime-t;


function [spcho]= pbip_numfct(M,spcho)
global cpu_numfct;
%
% Numerical factorization of matrix M. Previous call to symbfct
% required for a correct spcho cell array. 
% The last element of spcho (lnz) is updated and
% contains the factorization. It must be saved on return if necessary,
% i.e., if another numfct for another matrix with the same topology 
% but different values is going to be factorized with numfct.
% It calls ipnnv and blkfct routines of Ng-Peyton sparse cholesky package
% 
% Updates spcho with the last item lnz updated. On input this
% lnz item is any matrix (possibly empty), on output contains the 
% numerical factorization.
t= cputime;
m= size(M,1);
Mdiag = full(diag(M));
M= M - sparse(1:m,1:m,Mdiag);
if nnz(M) > 0 % don't factorize if matrix is diagonal
    [lnz] = inpnv(Mdiag,M,spcho{10},spcho{9},spcho{1},spcho{3},spcho{4},...
            spcho{5},spcho{2});
    level=8;
    [lnz] = blkfct(spcho{1},spcho{3},spcho{6},spcho{7},spcho{4},spcho{5},...
            lnz,spcho{8},level);
else
     lnz = Mdiag;
end
spcho{11}= lnz;
cpu_numfct= cpu_numfct+cputime-t;


function [sol]= pbip_solfct(rhs,spcho)
global cpu_solfct;
%
%  Solves M*sol = rhs, 
%  Previous calls to symbfct and numfct are assumed for a correct 
%  factorization, stored in spcho.
%  It calls blkslv routine of Ng-Peyton sparse cholesky package
%
%  Input:
%   - rhs: right-hand-side in original ordering
%   - spcho: cell array with factorization obtained after calls 
%     to symbfct and numfct
%  Output:
%   -sol: solution of system
%
t=cputime;
m= size(rhs,1);
sol= zeros(m,1);
if length(spcho{11}) == m        % diagonal solve
    % spcho{11} is lnz
    sol(spcho{11}>0) = rhs(spcho{11} > 0) ./ spcho{11}(spcho{11} > 0);
else
    % spcho{9} is perm
    sol(spcho{9}) = blkslv(spcho{1},spcho{3},spcho{4},spcho{5},spcho{11},rhs(spcho{9}));
end
cpu_solfct= cpu_solfct+cputime-t;


function [m,n,k,l,km,kn,ini_m,ini_n,q,activelnk]= ...
      pbip_preprocessing(c,q,u,b,N,L,activelnk,BIG)
global set_with_ub set_without_ub;
%
% Performs a simple preprocessing of the problem
%
% Output parameters:
% - m,n,k,l: number of constraints and variables per block (m,n), number
%   of blocks (k) and number of linking constraints (l). m, n arre scalars
%   or vectors depending on N is a list of matrices or a constant matrix
%   for all blocks.
%   km= sum{i=1:k}m_i or k*m ; kn= sum{i=1:k}n_i or k*n ;
% - ini_m, ini_n: position of first column/row of block i; i.e., rows
%   of block i go from ini_m(i) to ini_m(i+1)-1, and columns go from
%   ini_n(i) to ini_n(i+1)-1
% - q[kn+l]: set to 0 if initially empty matrix
% - activelnk[]: vector of indices of active linking constraints
%                if empty, assume active all linking constraints
%
% Simple check for
% - dimensions
% - no-zero upper bounds (variables and slacks) not allowed. Set a very small
%   upper bound to guarantee an interior solution. 
%   Zero variables are not removed
%   to preserve the topology of the constraints for each block
%   then we can maintain one single symbolic factorization.
% - q positive semidefinite; is set to 0 if problem is linear
% - initialization of activelnk if empty
%

% Dimensions
k= 0;
if (issparse(N)) 
    [m,n]= size(N);
    ini_m=[];
    ini_n=[];
elseif iscell(N)
    k= length(N);
    if (k==0) error('empty list of matrices: N'); end
    for i=1:k
        [m(i),n(i)]= size(N{i});
    end
    km= sum(m);
    kn= sum(n);
    ini_m= [1 cumsum(m)+1];
    ini_n= [1 cumsum(n)+1];
else
   error('expected sparse matrix or cell array: N'); 
end

if issparse(L)
    [l,nc]= size(L);    
    if (~isempty(L))
        if (iscell(N))
            for i= 1:k  
                if (nc ~= n(i)) 
                    error('wrong dimensions: L, block',i);
                end
            end
        else
            if (nc ~= n) 
                error('wrong dimensions: L');
            end
        end       
    end
elseif iscell(L)
    kL= length(L);
    if (kL==0) error('empty list of matrices: L'); end
    l= size(L{1},1);
    if iscell(N)
        if (k ~= kL) error('wrong number of matrices in L'); end
        for i=1:k
            [laux,ncaux]= size(L{i});
            if (laux ~= l) | (ncaux ~= n(i)) 
                error('wrong dimensions: L, block',i);
            end
        end
    else
        k= kL;
        km= k*m;
        kn= k*n;
        for i=1:k
            [laux,ncaux]= size(L{i});
            if (laux ~= l) | (ncaux ~= n) 
                error('wrong dimensions: L, block',i);
            end
        end
    end
else
   error('expected sparse matrix or cell array: L'); 
end
nc= size(c,1);
if (k>0)
    if (nc ~= kn+l)  error('wrong dimensions: c'); end
else
    k= (nc-l)/n; 
    if (k ~= round(k))  error('wrong dimensions: c'); end
    kn= k*n;
    km= k*m;
end

if (~isempty(q)) 
  if (nc ~= size(q,1))  error('wrong dimensions: q'); end
end
if (nc ~= size(u,1) )  error('wrong dimensions: u'); end
if ((km+l) ~= size(b,1))  error('wrong dimensions: b'); end
if (~isempty(activelnk))
    if (length(activelnk) > l)  
      error('wrong dimensions: activelnk'); 
    elseif ((any(activelnk < 1)) | (any(activelnk >l))) 
      error('wrong indices: activelnk');
    end
end
	
% No-zero upper bounds
if any(u < 1.0e-17)  error('zero lower bounds in u not allowed'); end

% q positive semidefinite; set to 0 if problem is linear
if (isempty(q))  q= zeros(kn+l,1); end
if (any(q<0.0))  error('q is not positive semidefinite'); end

% initialize activelnk if empty
if (isempty(activelnk))
  activelnk(1:l)= (1:l)'; 
end

% set of variables with and without upper bounds
set_with_ub= find(u < BIG);
set_without_ub= find(u>=BIG);


function [x, flag, err, iter, res] = my_pcg(A,b,tol,maxIter,M,x,varargin)
global cpu_my_pcg_precond;
% [x, flag, err, iter, res] = my_pcg(A, b, tol, maxIter, M, x, varargin)
%
% PCG solves the symmetric positive definite linear system Ax=b 
% using the Preconditionned Conjugate Gradient.
%
% input   A        REAL symmetric positive definite matrix or function
%         b        REAL right hand side vector
%         tol      REAL error tolerance (default: 1e-8)
%         maxIter  INTEGER maximum number of iterations (default: 50)
%         M        REAL preconditioner matrix (single matrix, or list of two
%                  matrices M1, M2 such that M= M1*M2 (default: none)
%         x        REAL initial guess vector
%         varargin parameters for A and M functions
%
% output  x        REAL solution vector
%         flag     INTEGER: 0 = solution found to tolerance
%                           1 = no convergence given maxIter
%         err      REAL final relative norm of the residual
%         iter     INTEGER number of iterations performed
%         res      REAL residual vector

%     Details of this algorithm are described in 
%
%     'Templates for the Solution of Linear Systems: Building Blocks 
%     for Iterative Methods', 
%     Barrett, Berry, Chan, Demmel, Donato, Dongarra, Eijkhout,
%     Pozo, Romine, and Van der Vorst, SIAM Publications, 1993
%     (ftp netlib2.cs.utk.edu; cd linalg; get templates.ps).
%
%     'Iterative Methods for Sparse Linear Systems, Second Edition'
%     Saad, SIAM Publications, 2003
%     (ftp ftp.cs.umn.edu; cd dept/users/saad/PS; get all_ps.zip).

% Sage Group (IRISA, 2004)
% Adapted by Jordi Castro, June 2005

% -----------------------
% Parsing input arguments
% -----------------------
lhs= nargout;
rhs= nargin;

if (rhs < 2),
  error('pcg: not enough input arguments');
end

% Parsing the matrix A (expect sparse matrix or function)
if (issparse(A))
    matrixType = 1;
else
   matrixType = 0;
end

% If A is a matrix (dense or sparse)
if (matrixType == 1),
  if (size(A,1) ~= size(A,2)),
    error('pcg: matrix A must be square');
  end
end

% Parsing right hand side b
if (size(b,2) ~= 1),
  error('pcg: right hand side b must be a column vector');
end
if (matrixType ==1),
  if (size(b,1) ~= size(A,1)),
    error('pcg: right hand side b must have the size of the matrix A');
  end 
end

% Parsing of the error tolerance tol
if (rhs >= 3),
   if (~isscalar(tol))
      error('pcg: tol must be a scalar');
   end
else
   tol=1e-8;
end

% Parsing of the maximum number of iterations max_it
if (rhs >= 4),
   if (~isscalar(maxIter))
      error('pcg: maxIter must be a scalar');
   end 
else
   maxIter = min(size(b,1),50);
end

% Parsing the preconditioner M (only expect sparse matrix, list of two 
% sparse matrices or function)
if (rhs >=5)
  if (issparse(M))
    precondType = 1;
  elseif (iscell(M))
    precondType = 3;
  else
    precondType = 2;
  end 
  if (precondType == 1),
    if (size(M,1) ~= size(M,2)),
      error('pcg: preconditionner matrix M must be square');
    end 
    if ( size(M,1) ~= size(b,1) ),
      error('pcg: preconditionner matrix M must have the size of b');
    end
  elseif (precondType == 3)
    if (size(M,2) ~= 2),
       error('pcg: preconditionner M must have two matrices M1, M2');
    end
     if (size(M{1},1) ~= size(M{1},2)),
      error('pcg: preconditionner matrix M1 must be square');
    end 
     if (size(M{2},1) ~= size(M{2},2)),
      error('pcg: preconditionner matrix M2 must be square');
    end 
    if ( size(M{1},1) ~= size(b,1) ),
      error('pcg: preconditionner matrix M1 must have the size of b');
    end
    if ( size(M{2},1) ~= size(b,1) ),
      error('pcg: preconditionner matrix M2 must have the size of b');
    end
  end   					       
else
  precondType = 0; %no preconditionner
end

% Parsing the initial vector x
if (rhs >= 6),
   if (size(x,2) ~= 1),
      error('pcg: initial guess x0 must be a column vector');
   end
   if (size(x,1) ~= size(b,1)),
     error('pcg: initial guess x0 must have the size of b');
   end 
else
  x=zeros(b);
end

% ------------
% Computations
% ------------


% initialization
flag = 0; 
bnrm2 = norm(b);
if  (bnrm2 == 0), 
  x = zeros(b);
  err = 0;
  iter = 0;
  res = err;
end

% r = b - A*x;
if (matrixType ==1),
  r = b - A*x;
else
  r = b - A(x,varargin{1:6});
end
err = norm(r) / bnrm2;
res = err;
if (err < tol),
  iter = 0;
  return; 
end

% begin iteration
for i = 1:maxIter,
  % z  = M \ r;
  if (precondType == 1),
    z = M \ r;
  elseif (precondType == 2),
    z = M(r,varargin{1:5});
  elseif (precondType == 3)
    taux= cputime;
    z = M{2} \ (M{1} \r); 
    cpu_my_pcg_precond= cputime-taux;
  else
    z = r;
  end
  rho = (r'*z);
  if (i > 1),
    bet = rho / rho_1;
    p = z + bet*p;
  else
    p = z;
  end
  
  % q = A*p;
  if (matrixType ==1),
    q = A*p;
  else
    q = A(p,varargin{1:6});
  end
  alp = rho / (p'*q );
  x = x + alp*p;
  r = r - alp*q;
% replace error standard computation by the angle criteria 
% suggested by Resende: b-r=Ax; if cos(Ax,b)~1 <=> Ax~b, we have the solution 
% Jordi Castro, June 2005
%
%  err = norm(r) / bnrm2; % standard criteria
 err= abs(1-abs((b-r)'*b)/(bnrm2*norm(b-r)));  % angle criteria
%
  res = [res;err];
  if (err <= tol),
    iter=i; 
    break;
  end 
  rho_1 = rho;
  if (i == maxIter ), 
    iter=i;
  end
end

% test for convergence
if (err > tol),
  flag = 1; 
  if (lhs < 2),
    warning('PCG did not converge');
  end
end

