function [x,obj]= nonor_qipm_linprog(c,q,uk,u,b,od,tsolver)
//
// [x,obj]= nonor_qipm_linprog(c,q,uk,u,b,od[,tsolver])
//
// NONOR_QIPM_LINPROG solves the linear multicommodity problem calling internal
// Scilab linpro or lipsol routine (you need to load lipsol first, if you
// want to use it)
// The input parameters are:
//
// c[n,k+1]:     linear forward arc costs; last column is for slacks of 
//               mutual capacity constraints. The same arc costs are
//               assumed for the backward arcs.
// q[n,k+1]:     quadratic forward arc costs; last column is for slacks of 
//               mutual capacity constraints (q=[] if no quadratic costs).
//               The same costs are assumed for backward arcs.
// uk[n,k+1]:    upper bounds (capacities) of forward arcs for each commodity; 
//               last column is for slacks of mutual capacity constraints.
//               The same capacities are assumed for backward arcs.
// u[n]:         mutual capacity of arcs for all commodities.
// b[m,k]:       supplies (positive values) and demands (negative values) 
//               for each commodity at nodes of the network
// od[n,2]:      for each forward arc, origin (first column) and destination 
//               (second column) nodes
// tsolver:      1 if linpro; 2 if lipsol; default is linpro
//
// Jordi Castro, June 2005
// Departament of Statistics and Operations Research
// Universitat Politcnica de Catalunya, Barcelona
// www-eio.upc.es/~jcastro, jordi.castro@upc.edu
//

//********** general stuff: check parameters, ... ***********
[lhs,rhs]= argn();
if (rhs < 6)
   error('too few input parameters');
end
if (rhs== 6) then tsolver= 1; end
if (~isempty(q))
  error('q is not empty, quadratic models can not be solved');
end

// ********* preprocess mutual capacity constraints ************
// uk is modified by qipm_preprocessing (only upper bounds of arcs)
// u is also modified
[m,n,k,uk,u,activecmt]= qipm_preprocessing(c,q,uk,u,b,od,1.0E+32);

// compute nonoriented node-arc incidence matrix N, excluding last row (flag~=0)
flag= 1;
N= qipm_nonor_network_matrix(od,flag);

// compute linking constraints structure matrix
L= qipm_nonor_linking_matrix(n);

// replicate costs, and capacities for backward arcs
for i=1:k
  c2(1:n,2*(i-1)+1)= c(:,i);
  c2(1:n,2*(i-1)+2)= c(:,i);
  if (~isempty(q))
      q2(1:n,2*(i-1)+1)= q(:,i);
      q2(1:n,2*(i-1)+2)= q(:,i);
  end
  uk2(1:n,2*(i-1)+1)= uk(:,i);
  uk2(1:n,2*(i-1)+2)= uk(:,i);
end
c2(1:n,2*k+1)= c(:,k+1);
if (~isempty(q))
    q2(1:n,2*k+1)= q(:,k+1);
end
uk2(1:n,2*k+1)= uk(:,k+1);

// empty_activecmt selects if activecmt= [] must be sent to prblock_ip
empty_activecmt= 1;

if (empty_activecmt) 
  // reshape problem matrices to vectors, avoiding injections of last node
  // and considering only active linking constraints (activecmt2= []);
  c2= [matrix(c2(1:n,1:2*k),k*2*n,1) ; c2(activecmt,2*k+1)];
  if (~isempty(q))  
    q2= [matrix(q2(1:n,1:2*k),k*2*n,1) ; q2(activecmt,2*k+1)];
  else
    q2= [];
  end
  u2= [matrix(uk2(1:n,1:2*k),k*2*n,1); uk2(activecmt,2*k+1)];
  b2= [matrix(b(1:m-1,1:k),k*(m-1),1); u(activecmt)];
  L2=L(activecmt,:);
  activecmt2= [];
else 
  // reshape problem matrices to vectors, avoiding injections of last node
  // and considering all linking constraints
  c2= matrix(c2,k*2*n+n,1);
  if (~isempty(q))  
    q2= matrix(q,k*2*n+n,1);
  else
    q2= [];
  end
  u2= matrix(uk,k*2*n+n,1);
  b2= [matrix(b(1:m-1,1:k),k*(m-1),1); u];
  L2= L;
  activecmt2= activecmt;
end

// solve problem through linpro or lipsol
l= size(L2,1);
A= get_A(N,L2,k,m-1,2*n,l);
if (tsolver== 1)
  [x2,lagr,obj]=linpro(c2,A,b2,zeros(2*k*n+l,1),u2,size(A,1))
else
  [x2,obj] = lipsol(A,b2,c2,zeros(2*k*n+l,1),u2,1.0E+32,1)
end


// reshape solution vectors, acording to empty_activecmt
// adding positions of y associated to last node with a 0 value

if (empty_activecmt) 
  // only positions of active linking constraints were considered
  x= matrix(x2(1:2*n*k),n,2*k);
  x(1:n,2*k+1)= 0;
  x(activecmt,2*k+1)= x2(2*k*n+1:2*k*n+l);
else
  x= matrix(x2,n,2*k+1);
end
 

function [m,n,k,uk,u,activecmt]= qipm_preprocessing(c,q,uk,u,b,od,BIG)
//
// Performs a simple preprocessing of the problem (see below an explanation)
//
// Output parameters:
// - m,n,k: number of nodes, nonoriented arcs and commodities
// - uk[m,k+1]: updated vector of capacities (only changes uk(1:m,1:k)
// - activecmt(): vector with the indices of active mutual capacity
//   constraints; indices correspond to the relative numeration of
//   mutual capacity constraints: 1:n
//
// Simple check for
// - dimensions
// - no-zero upper bounds (arcs and slacks) not allowed. Set a very small
//   upper bound to guarantee an interior solution. Zero arcs are not removed
//   to preserve he topology of the network for all the commodities,
//   then we can maintain one single symbolic factorization.
// - nodes in network topology between bounds
//
// Simple preprocessing for
// - infinity capacities, replaced by an upper bound of the flow in the arc.
//   This has to be done BEFORE preprocessing active linking constraints, since
//   infinity capacities might reduce the number of inactive constraints.
// - linking constraints: early detection of inactive 
//   constraints; they are dinamycally updated later during the optimization
//   in order to reduce the size of the system to be solved by PCG

//Dimensions
[n,k1]= size(c);
k= k1-1;
if (~isempty(q)) 
  [nr,nc]= size(q);
  if (nr ~= n ) | (nc ~= k1) then error('inconsistent dimensions: q'); end
end
[nr,nc]= size(uk);
if (nr ~= n ) | (nc ~= k1) then error('inconsistent dimensions: uk'); end
[nr]= size(u);
if (nr ~= n ) then error('inconsistent dimensions: u'); end
[m,nc]= size(b);
if (nc ~= k ) then error('inconsistent dimensions: b'); end
[nr,nc]= size(od);
if (nr ~= n ) | (nc ~= 2) then error('inconsistent dimensions: od'); end

// Network topology
if or(od<1 | od>m) then error('node<1 or node>m  in network topology'); end

// zero capacities replaced by small value (EPS_CAP)
EPS_CAP= 1.0e-16;
uk(uk<EPS_CAP)= EPS_CAP;
u(u<EPS_CAP)= EPS_CAP;

// Infinity capacities are replaced by the maximum of the maximum capacity
// (excluding infinity capacities) and an upper bound of the flow at each arc.
// This can provide a wrong problem for networks with negative costs, 
// infinity capacities, and negative cycles. These problems should be avoided.
// This has to be done BEFORE detecting inactive mutual capacity constraints,
// because the original infinity capacity might reduce the number of
// inactive constraints.
maxflow= sum(b(b>0));
// epsilon used when u()= BIG (i.e., k=1 and all linking constraints inactive)
EPSILON= 1.0e-10;
maxcap= max(max(uk(uk(:,1:k)<BIG)),max([u(u<BIG);EPSILON]));
newinfcap= max(maxflow,maxcap);
uk(uk(:,1:k) >= BIG)= newinfcap;

// Early detection of inactive linking constraints, checking whether:
// - sum of capacities for all commodities lower than mutual capacity 
// We don't treat the particular case where k=1 (single commodity problem).
// In that case the mutual capacity u() and the upper bound of slacks
// uk(:,k+1) should be set to BIG by the user, and the slacks linear and
// quadratic costs to 0. That way, all the mutual capacity constraints
// will be automatically considered inactive, and removed.
//
// This check is only performed if:
// 1. the linking constraint of the arc has
// only upper bounds u and the default lower bound is 0, i.e., if the upper 
// bound of the slack is equal to u. 
// 2. the slack of the linking constraint does not appear in the objective 
// function; otherwise, removing this constraint (and slack) can provide
// bad function evaluations. 
// docheck is used for this purpose.
if (isempty(q)) then
   docheck= (u==uk(:,k+1)) & (c(:,k+1)== 0);
else
   docheck= (u==uk(:,k+1)) & (c(:,k+1)== 0) & (q(:,k+1)== 0);
end
// The list of active linking mutual capacity constraints is maintained by 
// vector activecmt(i)= True if i active, False if i inactive, i=1..n
// We compute an equivalent vector inactivecmt for inactive constraints, 
// and complement it at the end
//
// Check if sum of capacities of all commodities is lower than mutual capacity
// or if mutual capacity is greater or equal than BIG
cap_all_comm= sum(uk(:,1:k),2);
inactivecmt= ( (cap_all_comm<=u) & docheck)| ((u>=BIG) & docheck);
activecmt= (find(~inactivecmt))';
 

function [N]= qipm_nonor_network_matrix(od,flag)
//
// Creates the nonoriented node-arc incidence matrix
// 
// Input parameters:
// -od[1:n,2]: origin and destination for each arc
// -flag: if is not zero, we remove the last node (last row), providing a 
//  full row rank matrix; if is zero, a rank defficient matrix that includes
//  last row is returned.

// Output parameters:
// - N: sparse matrix with nonoriented node-arc incidence network matrix

// compute number of arcs n, and number of nodes
// it is assumed that the number of nodes is the maximum value in od
[n,nc]= size(od);
m= max(max(od));

// ij[1:n,1:2] store the location (row,col) of the +1
ij(1:n,2)= [1:n]';
ij(1:n,1)= od(1:n,1);
// ij[n+1:2*n,1:2] store the location (row,col) of the -1
ij(n+1:2*n,2)= [1:n]';
ij(n+1:2*n,1)= od(1:n,2);
// v[1:2*n] store the values
v(1:n)= +1;
v(n+1:2*n)= -1;
// create the network
N= sparse(ij,v); 
if (flag ~=0) 
  N= N(1:m-1,1:n);
end
N= [N -N]; 

function [L]= qipm_nonor_linking_matrix(n)
//
// Creates the linking matrix constraints. For nonoriented multicommodity flows
// is [I I] where I is the the identity matrix of dimension n
// The input parameter:
// n: number of arcs
// 
L= speye(n,n);
L= [L L];


function [A]= get_A(N,L,k,m,n,l)
//
// Creates the full system matrix provided the block constraints N,
// and linking constraints L.
//
A= sparse([],[],[k*m+l,k*n+l]);
for i=1:k
  A((i-1)*m+1:i*m,(i-1)*n+1:i*n)= N; // diagonal blocks
  A(k*m+1:k*m+l,(i-1)*n+1:i*n)= L; // linking costraints 
end
A(k*m+1:k*m+l,k*n+1:k*n+l)= speye(l,l); // slacks of linking constraints


function [b]= isempty(a)
b= (a==[]);

function [b]= isscalar(a)
b= and(size(a) == [1 1]);
