/****************************************************************************/
// Program to convert a multicommodity instance in pprn format to 
// a matlab/scilab/octave matrix format. 
// It creates a .m file with the same name of the input pprn_file
// The output file is directly read by by scilab (typing "exec('name.m')"),
// or by matlab/octave (typing "name")
//
// usage: pprn2mat pprn_file
//
// It is assumed that exponents in the pprn file are in the C++ style 
// E+/E-/e+/e-, instead of the fortran style D+/D-/d+/d-. If not guaranteed,
// the output file will likely be wrong.
//
// The vectors and matrices in the output file are described below, using the
// following dimensions:
// 
// m is the number of nodes in network
// n is number of arcs in network
// k is number of commodities
// l is number of side constraints
// nnzl is the number of non-zero elements in side constraints
//
// c[n,k+1]:     linear arc costs; last column is for slacks of mutual capacity
//               constraints
// q[n,k+1]:     quadratic arc costs; last column is for slacks of mutaul 
//               capacity constraints (q=[] if no quadratic costs)
// uk[n,k+1]:    upper bounds (capacities) of arcs for each commodity; 
//               last column is for slacks of mutual capacity constraints 
// u[n]:         mutual capacity of arcs for all commodities
// b[m,k]:       supplies (positive values) and demands (negative values) 
//               for each commodity at nodes of the network
// od[n,2]:      for each arc, origin (first column) and destination 
//               (second column) nodes
// lu_sc[l,2]:   lower and upper bounds for each side constraint 
//               (first column is for lower, second column for upper)
//               lu_sc=[] if no side constraints.
// nnz_sc[nnz,4]:structure of nonzero values of side constraints. Each row 
//               gives the position (first three columns, providing the arc, 
//               commodity, and side constraint) and value (last column)
//               of the nonzero. nnz_sc=[] if no side constraints.
//
// Jordi Castro, 19-5-2005    
/****************************************************************************/
#include <iostream>
#include <iomanip>
#include <fstream>
#include <vector>
#include <string>

using namespace std;

void inline skip_line(ifstream& f)
{
    const int MAXLINE= 1000000;
    f.ignore(MAXLINE,'\n');
}

int main(int argc, char *argv[])
{  
    
    // check number of command line arguments
    if (argc !=2 ) {
	cerr << "usage: pprn2mat pprn_file \n";
	exit(-1);
    }
    
    // deal with names of files and open files
    string namepprnfile(argv[1]);
    string namematfile;
    int posdot = namepprnfile.find("."); //assume posdot negative if not found
    namematfile= namepprnfile.substr(0,posdot)+".m";

    ifstream pprnfile(namepprnfile.c_str());
    if (!pprnfile) {
	cerr << "Can not open input file " << namepprnfile << "\n";
	exit(-1);
    }
    ofstream matfile(namematfile.c_str());
    if (!matfile) {
	cerr << "Can not open output file " << namematfile << "\n";
	exit(-1);
    }
    matfile.precision(16); //default digits for floating numbers in output file

    // reading pprn file and writing matrix file

    const int num_per_line= 5; //maximum numbers per line in output file

    // problem dimensions
    int m, // number of nodes
	n,  // number of arcs
	k,  // number of commodities
	l, // number of linear side constraints
	nnz, // upper bound of number of non-zero side constraints coefficients
	nq; // if positive, there are quadratic costs for each arc  
    pprnfile >> m >> n >> k >> l >> nnz >> nq ;
//     matfile << "m= " << m << ";\n";
//     matfile << "n= " << n << ";\n";
//     matfile << "k= " << k << ";\n";
//     matfile << "l= " << l << ";\n";
    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line
    
    // linear costs per commodity
    double r;
    int ct= 0;
    matfile << "c = [ \n";
    for(int h=0;h<k;h++) {
	for(int i=0;i<n;i++) {
	    pprnfile >> r;
	    matfile << r << " ";
	    if (++ct == num_per_line) {
		matfile << " ... \n";
		ct= 0;
	    }
	}
	if (h != k-1) matfile << "; \n";
    }
    matfile << "]'; \n";
    /* add slacks default costs, currently 0 in pprn format; in the 
       future we can consider nonzero slacks costs  */
    matfile << "c(:,"<<k<<"+1) = zeros("<<n<<",1);\n"; 

    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line
    
    // quadratic costs per commodity
    if (nq > 0) {
	ct= 0;
	matfile << "q = [ \n";
	for(int h=0;h<k;h++) {
	    for(int i=0;i<n;i++) {
		pprnfile >> r;
		matfile << r << " ";
		if (++ct == num_per_line) {
		    matfile << " ... \n";
		    ct= 0;
		}		
	    }
	    if (h != k-1) matfile << "; \n";
	}
	matfile << "]'; \n";
    /* add slacks default costs, currently 0 in pprn format; in the 
       future we can consider nonzero slacks costs  */
	matfile << "q(:,"<<k<<"+1)= zeros("<<n<<",1);\n"; 	
	skip_line(pprnfile); //end current line
	skip_line(pprnfile); //skip next line
    }
    else
	matfile << "q = []; \n";

    // arc capacities per commodity
    ct= 0;
    matfile << "uk = [\n ";
    for(int h=0;h<k;h++) {
	for(int i=0;i<n;i++) {
	    pprnfile >> r;
	    matfile << r << " ";
	    if (++ct == num_per_line) {
		matfile << " ... \n";
		ct= 0;
	    }
	}
	if (h != k-1) matfile << "; \n";
    }
    matfile << "]'; \n";
    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line

    // node supply/demands per commodity
    ct= 0;
    matfile << "b = [ \n";
    for(int h=0;h<k;h++) {
	for(int i=0;i<m;i++) {
	    pprnfile >> r;
	    matfile << setprecision(20) << r << " ";
	    if (++ct == num_per_line) {
		matfile << " ... \n";
		ct= 0;
	    }
	}
	if (h != k-1) matfile << "; \n";
    }
    matfile << "]'; \n";
    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line
     
    // mutual arc capacities for all the commodities
    ct= 0;
    matfile << "u = [ \n";
    for(int i=0;i<n;i++) {
	    pprnfile >> r;
	    matfile << r << " ";
	    if (++ct == num_per_line) {
		matfile << " ... \n";
		ct= 0;
	    }
	}
    matfile << "]'; \n";
    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line
    // add slacks deffault upper bounds, mutual capacities in pprn files
    matfile << "uk(:,"<<k<<"+1)= u;\n"; 

    // network topology: origin-destination nodes for each arc
    int o, d;
    ct= 0;
    matfile << "od= [ \n";
    for(int i=0;i<n;i++) {
	pprnfile >> o >> d;
	matfile << o << " " << d << "\n";
    }
    matfile << "]; \n";
    skip_line(pprnfile); //end current line
    skip_line(pprnfile); //skip next line

    // side constraints structure
    if (l>0) {
	// lower and upper bounds of side constraints
	ct= 0;
	matfile << "lu_sc= [ \n";
	for(int j=0; j<2; j++) { // j=0 for lower j=1 for upper bounds
	    for(int i=0;i<l;i++) {
		pprnfile >> r;
		matfile << r << " ";
		if (++ct == num_per_line) {
		    matfile << " ... \n";
		    ct= 0;
		}
	    }
	    if (j==0) matfile << "; \n"; 
	    skip_line(pprnfile); //end current line
	}
	matfile << "]'; \n";
	skip_line(pprnfile); //skip next line

	// nonzero coefficients of side constraints
	ct=0;
	int la,lk,lsc;
	matfile << "nnz_sc= [\n ";	
	for(int i=0;(i<nnz)&&(pprnfile >> la >> lk >> lsc >> r);i++) {
	    matfile << la << " " << lk << " " << lsc << " "<< r << "\n";
	}
	matfile << "]; \n";
    }
    else {
	matfile << "lu_sc = []; \n";
	matfile << "nnz_sc = []; \n";
    }

    pprnfile.close();
    matfile.close();
}
